@@ -6,6 +6,9 @@ from os.path import join
 
 def configuration(parent_package='',top_path=None):
     from numpy.distutils.misc_util import Configuration
+    from numpy.distutils.system_info import get_info
+
+    lapack_opt = get_info('lapack_opt', notfound_action=2)
 
     config = Configuration('interpolate', parent_package, top_path)
 
@@ -15,6 +18,10 @@ def configuration(parent_package='',top_path=None):
     config.add_extension('interpnd',
                          sources=['interpnd.c'])
 
+    config.add_extension('_ppoly',
+                         sources=['_ppoly.c'],
+                         **lapack_opt)
+
     config.add_extension('_fitpack',
                          sources=['src/_fitpackmodule.c'],
                          libraries=['fitpack'],
