@@ -11,35 +11,21 @@ is granted under the SciPy License.
 
 #include "sigtools.h"
 #include <setjmp.h>
+#include <stdlib.h>
 
 #define PYERR(message) {PyErr_SetString(PyExc_ValueError, message); goto fail;}
 
-#define DATA(arr) ((arr)->data)
-#define DIMS(arr) ((arr)->dimensions)
-#define STRIDES(arr) ((arr)->strides)
-#define ELSIZE(arr) ((arr)->descr->elsize)
-#define OBJECTTYPE(arr) ((arr)->descr->type_num)
-#define BASEOBJ(arr) ((PyArrayObject *)((arr)->base))
-#define RANK(arr) ((arr)->nd)
-#define ISCONTIGUOUS(m) ((m)->flags & CONTIGUOUS)
-
 
 jmp_buf MALLOC_FAIL;
 
-char *check_malloc (int);
-
-char *check_malloc (size)
-	int size;
+char *check_malloc(size_t size)
 {
-    char *the_block;
-    
-    the_block = (char *)malloc(size);
-    if (the_block == NULL)
-	{
-	    printf("\nERROR: unable to allocate %d bytes!\n", size);
-	    longjmp(MALLOC_FAIL,-1);
-	}
-    return(the_block);
+    char *the_block = malloc(size);
+    if (the_block == NULL) {
+        printf("\nERROR: unable to allocate %zu bytes!\n", size);
+        longjmp(MALLOC_FAIL,-1);
+    }
+    return the_block;
 }
 
 
@@ -70,7 +56,9 @@ static int index_out_of_bounds(intp *indices, intp *max_indices, int ndims) {
  * but probably with dim1 being the size of the "original, unsliced" array
  */
 
-static intp compute_offsets (uintp *offsets, intp *offsets2, intp *dim1, intp *dim2, intp *dim3, intp *mode_dep, int nd) {
+static intp compute_offsets (uintp *offsets, intp *offsets2, intp *dim1,
+                             intp *dim2, intp *dim3, intp *mode_dep,
+                             int nd) {
   int k,i;
   intp init_offset = 0;
 
@@ -573,7 +561,9 @@ static double wate(double freq, double *fx, double *wtx, int lband, int jtype)
  *  Adpated from main() by Travis Oliphant
  */
 
-static int pre_remez(double *h2, int numtaps, int numbands, double *bands, double *response, double *weight, int type, int maxiter, int grid_density) {
+static int pre_remez(double *h2, int numtaps, int numbands, double *bands,
+                     double *response, double *weight, int type, int maxiter,
+                     int grid_density) {
   
   int jtype, nbands, nfilt, lgrid, nz;
   int neg, nodd, nm1;
@@ -648,7 +638,11 @@ static int pre_remez(double *h2, int numtaps, int numbands, double *bands, doubl
 	    temp = grid[j];
 	    des[j] = eff(temp,fx,lband,jtype);
 	    wt[j] = wate(temp,fx,wtx,lband,jtype);
-	    if (++j > wrksize) { free(tempstor); return -1;} /* too many points, or too dense grid */
+	    if (++j > wrksize) {
+                /* too many points, or too dense grid */
+                free(tempstor);
+                return -1;
+            } 
 	    grid[j] = temp + delf;
 	} while (grid[j] <= fup);
 
@@ -703,7 +697,11 @@ static int pre_remez(double *h2, int numtaps, int numbands, double *bands, doubl
     nm1 = nfcns - 1;
     nz  = nfcns + 1;
 
-    if (remez(&dev, des, grid, edge, wt, ngrid, numbands, iext, alpha, nfcns, maxiter, work, dimsize) < 0) { free(tempstor); return -1; }
+    if (remez(&dev, des, grid, edge, wt, ngrid, numbands, iext, alpha, nfcns,
+              maxiter, work, dimsize) < 0) {
+        free(tempstor);
+        return -1;
+    }
 
     /*
      * CALCULATE THE IMPULSE RESPONSE.
@@ -767,15 +765,17 @@ static int pre_remez(double *h2, int numtaps, int numbands, double *bands, doubl
 /* N-D Order Filtering. */
 
 
-static void fill_buffer(char *ip1, PyArrayObject *ap1, PyArrayObject *ap2, char *sort_buffer, int nels2, int check, intp *loop_ind, intp *temp_ind, uintp *offset){ 
+static void fill_buffer(char *ip1, PyArrayObject *ap1, PyArrayObject *ap2,
+                        char *sort_buffer, int nels2, int check,
+                        intp *loop_ind, intp *temp_ind, uintp *offset){ 
   int i, k, incr = 1;
-  int ndims = ap1->nd;
-  intp *dims2 = ap2->dimensions;
-  intp *dims1 = ap1->dimensions;
-  intp is1 = ap1->descr->elsize;
-  intp is2 = ap2->descr->elsize;
-  char *ip2 = ap2->data;
-  int elsize = ap1->descr->elsize;
+  int ndims = PyArray_NDIM(ap1);
+  intp *dims2 = PyArray_DIMS(ap2);
+  intp *dims1 = PyArray_DIMS(ap1);
+  intp is1 = PyArray_ITEMSIZE(ap1);
+  intp is2 = PyArray_ITEMSIZE(ap2);
+  char *ip2 = PyArray_DATA(ap2);
+  int elsize = PyArray_ITEMSIZE(ap1);
   char *ptr;
 
   i = nels2;
@@ -792,11 +792,12 @@ static void fill_buffer(char *ip1, PyArrayObject *ap1, PyArrayObject *ap2, char
     temp_ind[k]++;
 
     if (!(check && index_out_of_bounds(temp_ind,dims1,ndims)) && \
-	memcmp(ip2, ptr, ap2->descr->elsize)) { 
+	memcmp(ip2, ptr, PyArray_ITEMSIZE(ap2))) {
       memcpy(sort_buffer, ip1, elsize);
       sort_buffer += elsize;
     } 
-    incr = increment(loop_ind, ndims, dims2);  /* Returns number of N-D indices incremented. */
+    /* Returns number of N-D indices incremented. */
+    incr = increment(loop_ind, ndims, dims2);  
     ip2 += is2;
 
   }
@@ -861,51 +862,55 @@ PyObject *PyArray_OrderFilterND(PyObject *op1, PyObject *op2, int order) {
 	ap2 = (PyArrayObject *)PyArray_ContiguousFromObject(op2, typenum, 0, 0);
 	if (ap2 == NULL) goto fail;
 
-	if (ap1->nd != ap2->nd) {
-	  PyErr_SetString(PyExc_ValueError, "All input arrays must have the same number of dimensions.");
+	if (PyArray_NDIM(ap1) != PyArray_NDIM(ap2)) {
+	    PyErr_SetString(PyExc_ValueError,
+                "All input arrays must have the same number of dimensions.");
 	  goto fail;
 	}
 
 	n2 = PyArray_Size((PyObject *)ap2);
 	n2_nonzero = 0;
-	ap2_ptr = ap2->data;
-	/* Find out the number of non-zero entries in domain (allows for
-	 *  different shapped rank-filters to be used besides just rectangles)
-	 */
+	ap2_ptr = PyArray_DATA(ap2);
+        /*
+         * Find out the number of non-zero entries in domain (allows for
+         * different shapped rank-filters to be used besides just rectangles)
+         */
 	zptr = PyArray_Zero(ap2);
 	if (zptr == NULL) goto fail;
 	for (k=0; k < n2; k++) {
-	  n2_nonzero += (memcmp(ap2_ptr,zptr,ap2->descr->elsize) != 0);
-	  ap2_ptr += ap2->descr->elsize;
+	  n2_nonzero += (memcmp(ap2_ptr,zptr,PyArray_ITEMSIZE(ap2)) != 0);
+	  ap2_ptr += PyArray_ITEMSIZE(ap2);
 	}
 
 	if ((order >= n2_nonzero) || (order < 0)) {
-	  PyErr_SetString(PyExc_ValueError, "Order must be non-negative and less than number of nonzero elements in domain.");
+	    PyErr_SetString(PyExc_ValueError,
+                "Order must be non-negative and less than number of nonzero elements in domain.");
 	  goto fail;
 	}
 	
-	ret = (PyArrayObject *)PyArray_SimpleNew(ap1->nd, ap1->dimensions, typenum);
+	ret = (PyArrayObject *)PyArray_SimpleNew(PyArray_NDIM(ap1),
+                                                 PyArray_DIMS(ap1),
+                                                 typenum);
 	if (ret == NULL) goto fail;
 	
-	compare_func = compare_functions[ap1->descr->type_num];
+	compare_func = compare_functions[PyArray_TYPE(ap1)];
 	if (compare_func == NULL) {
-		PyErr_SetString(PyExc_ValueError, 
-			"order_filterND not available for this type");
+	    PyErr_SetString(PyExc_ValueError, 
+                            "order_filterND not available for this type");
 		goto fail;
 	}
 
-	is1 = ap1->descr->elsize;
+	is1 = PyArray_ITEMSIZE(ap1);
 	
 	if (!(sort_buffer = malloc(n2_nonzero*is1))) goto fail;
 
-	op = ret->data; os = ret->descr->elsize;
+	os = PyArray_ITEMSIZE(ret);
+	op = PyArray_DATA(ret);
 
-	op = ret->data;
-
-	bytes_in_array = ap1->nd*sizeof(intp);
+	bytes_in_array = PyArray_NDIM(ap1)*sizeof(intp);
 	mode_dep = malloc(bytes_in_array);
-	for (k = 0; k < ap1->nd; k++) { 
-	  mode_dep[k] = -((ap2->dimensions[k]-1) >> 1);
+	for (k = 0; k < PyArray_NDIM(ap1); k++) { 
+	  mode_dep[k] = -((PyArray_DIMS(ap2)[k]-1) >> 1);
 	}	
 
 	b_ind = (intp *)malloc(bytes_in_array);  /* loop variables */
@@ -915,9 +920,11 @@ PyObject *PyArray_OrderFilterND(PyObject *op1, PyObject *op2, int order) {
 	memset(ret_ind,0,bytes_in_array);
 	temp_ind = (intp *)malloc(bytes_in_array);
 	check_ind = (intp*)malloc(bytes_in_array);
-	offsets = (uintp *)malloc(ap1->nd*sizeof(uintp));
-	offsets2 = (intp *)malloc(ap1->nd*sizeof(intp));
-	offset1 = compute_offsets(offsets,offsets2,ap1->dimensions,ap2->dimensions,ret->dimensions,mode_dep,ap1->nd);
+	offsets = (uintp *)malloc(PyArray_NDIM(ap1)*sizeof(uintp));
+	offsets2 = (intp *)malloc(PyArray_NDIM(ap1)*sizeof(intp));
+	offset1 = compute_offsets(offsets, offsets2, PyArray_DIMS(ap1),
+                                  PyArray_DIMS(ap2), PyArray_DIMS(ret),
+                                  mode_dep, PyArray_NDIM(ap1));
 	/* The filtering proceeds by looping through the output array
 	   and for each value filling a buffer from the 
 	   element-by-element product of the two input arrays.  The buffer
@@ -939,22 +946,27 @@ PyObject *PyArray_OrderFilterND(PyObject *op1, PyObject *op2, int order) {
 	PyDataMem_FREE(zptr);
 	zptr = PyArray_Zero(ap1);
 	if (zptr == NULL) goto fail;
-	ap1_ptr = ap1->data + offset1*is1;
-	for (k=0; k < ap1->nd; k++) {a_ind[k] = mode_dep[k]; check_ind[k] = ap1->dimensions[k] - ap2->dimensions[k] - mode_dep[k] - 1;}
-	a_ind[ap1->nd-1]--;
+	ap1_ptr = (char *)PyArray_DATA(ap1) + offset1*is1;
+	for (k=0; k < PyArray_NDIM(ap1); k++) {
+            a_ind[k] = mode_dep[k];
+            check_ind[k] = PyArray_DIMS(ap1)[k] - PyArray_DIMS(ap2)[k] - mode_dep[k] - 1;
+        }
+	a_ind[PyArray_NDIM(ap1)-1]--;
 	i = PyArray_Size((PyObject *)ret);
 	while (i--) {
-	  /* Zero out the sort_buffer (has effect of zero-padding
-	     on boundaries). Treat object arrays right.*/
+          /*
+           * Zero out the sort_buffer (has effect of zero-padding
+           * on boundaries). Treat object arrays right.
+           */
 	  ap2_ptr = sort_buffer;
 	  for (k=0; k < n2_nonzero; k++) {
   	    memcpy(ap2_ptr,zptr,is1);
 	    ap2_ptr += is1;
 	  }
 	    
-	  k = ap1->nd - 1;
+	  k = PyArray_NDIM(ap1) - 1;
 	  while(--incr) {
-	    a_ind[k] -= ret->dimensions[k] - 1;   /* Return to start */
+	    a_ind[k] -= PyArray_DIMS(ret)[k] - 1;   /* Return to start */
 	    k--;
 	  }
 	  ap1_ptr += offsets2[k]*is1;
@@ -962,15 +974,18 @@ PyObject *PyArray_OrderFilterND(PyObject *op1, PyObject *op2, int order) {
 	  memcpy(temp_ind, a_ind, bytes_in_array);
 
 	  check = 0; k = -1;
-	  while(!check && (++k < ap1->nd))
-	    check = check || (ret_ind[k] < -mode_dep[k]) || (ret_ind[k] > check_ind[k]);
+	  while(!check && (++k < PyArray_NDIM(ap1)))
+	    check = (check || (ret_ind[k] < -mode_dep[k]) || 
+                     (ret_ind[k] > check_ind[k]));
 
 	  fill_buffer(ap1_ptr,ap1,ap2,sort_buffer,n2,check,b_ind,temp_ind,offsets);
 	  qsort(sort_buffer, n2_nonzero, is1, compare_func);
 	  memcpy(op, sort_buffer + order*is1, os);
-
-	  incr = increment(ret_ind,ret->nd,ret->dimensions); /* increment index counter */
-	  op += os;   /* increment to next output index */
+          
+          /* increment index counter */
+	  incr = increment(ret_ind,PyArray_NDIM(ret),PyArray_DIMS(ret)); 
+          /* increment to next output index */
+	  op += os;
 
 	}
 	free(b_ind); free(a_ind); free(ret_ind);
@@ -1001,7 +1016,8 @@ static char doc_correlateND[] = "out = _correlateND(a,kernel,mode) \n\n   mode =
 
 static char doc_convolve2d[] = "out = _convolve2d(in1, in2, flip, mode, boundary, fillvalue)";
 
-extern int pylab_convolve_2d(char*,intp*,char*,intp*,char*,intp*,intp*,intp*,int,char*);
+extern int pylab_convolve_2d(char*, intp*, char*, intp*, char*, intp*, intp*,
+                             intp*, int, char*);
 
 static PyObject *sigtools_convolve2d(PyObject *NPY_UNUSED(dummy), PyObject *args) {
 
@@ -1013,7 +1029,8 @@ static PyObject *sigtools_convolve2d(PyObject *NPY_UNUSED(dummy), PyObject *args
     PyArrayObject *ain1=NULL, *ain2=NULL, *aout=NULL;
     PyArrayObject *afill=NULL, *newfill=NULL;
 
-    if (!PyArg_ParseTuple(args, "OO|iiiO", &in1, &in2, &flip, &mode, &boundary, &fill_value)) {
+    if (!PyArg_ParseTuple(args, "OO|iiiO", &in1, &in2, &flip, &mode, &boundary,
+                          &fill_value)) {
         return NULL;
     }
 
@@ -1028,7 +1045,8 @@ static PyObject *sigtools_convolve2d(PyObject *NPY_UNUSED(dummy), PyObject *args
       PYERR("Incorrect boundary value.");
     if (boundary == PAD) {
 	if (fill_value == NULL) {
-	    newfill = (PyArrayObject *)PyArray_SimpleNewFromData(0, dims, typenum, zeros);
+	    newfill = (PyArrayObject *)PyArray_SimpleNewFromData(0, dims, typenum,
+                                                                 zeros);
 	}
 	else {
 	    afill = (PyArrayObject *)PyArray_FromObject(fill_value, NPY_CDOUBLE, 0, 0);
@@ -1042,22 +1060,28 @@ static PyObject *sigtools_convolve2d(PyObject *NPY_UNUSED(dummy), PyObject *args
 	if (newfill == NULL) goto fail;
     }
 
-    aout_dimens = malloc(ain1->nd*sizeof(intp));
+    aout_dimens = malloc(PyArray_NDIM(ain1)*sizeof(intp));
     switch(mode & OUTSIZE_MASK) {
     case VALID:
-	for (i = 0; i < ain1->nd; i++) { 
-	    aout_dimens[i] = ain1->dimensions[i] - ain2->dimensions[i] + 1;
+	for (i = 0; i < PyArray_NDIM(ain1); i++) { 
+	    aout_dimens[i] = PyArray_DIMS(ain1)[i] - PyArray_DIMS(ain2)[i] + 1;
 	    if (aout_dimens[i] < 0) {
-		PyErr_SetString(PyExc_ValueError, "no part of the output is valid, use option 1 (same) or 2 (full) for third argument");
+		PyErr_SetString(PyExc_ValueError,
+                    "no part of the output is valid, use option 1 (same) or 2 "
+                    "(full) for third argument");
 		goto fail;
 	    }
 	}
 	break;
     case SAME:
-	for (i = 0; i < ain1->nd; i++) { aout_dimens[i] = ain1->dimensions[i];}
+	for (i = 0; i < PyArray_NDIM(ain1); i++) {
+            aout_dimens[i] = PyArray_DIMS(ain1)[i];
+        }
 	break;
     case FULL:
-	for (i = 0; i < ain1->nd; i++) { aout_dimens[i] = ain1->dimensions[i] + ain2->dimensions[i] - 1;}
+	for (i = 0; i < PyArray_NDIM(ain1); i++) {
+            aout_dimens[i] = PyArray_DIMS(ain1)[i] + PyArray_DIMS(ain2)[i] - 1;
+        }
 	break;
     default: 
 	PyErr_SetString(PyExc_ValueError, 
@@ -1065,22 +1089,23 @@ static PyObject *sigtools_convolve2d(PyObject *NPY_UNUSED(dummy), PyObject *args
 	goto fail;
     }
 	
-    aout = (PyArrayObject *)PyArray_SimpleNew(ain1->nd, aout_dimens, typenum);
+    aout = (PyArrayObject *)PyArray_SimpleNew(PyArray_NDIM(ain1), aout_dimens,
+                                              typenum);
     if (aout == NULL) goto fail;
 
     flag = mode + boundary + (typenum << TYPE_SHIFT) + \
       (flip != 0) * FLIP_MASK;
     
-    ret = pylab_convolve_2d (DATA(ain1),      /* Input data Ns[0] x Ns[1] */
-		             STRIDES(ain1),   /* Input strides */
-		             DATA(aout),      /* Output data */
-		             STRIDES(aout),   /* Ouput strides */
-		             DATA(ain2),      /* coefficients in filter */
-		             STRIDES(ain2),   /* coefficients strides */ 
-		             DIMS(ain2),      /* Size of kernel Nwin[2] */
-			     DIMS(ain1),      /* Size of image Ns[0] x Ns[1] */
-		             flag,            /* convolution parameters */
-		             DATA(newfill));  /* fill value */
+    ret = pylab_convolve_2d (PyArray_DATA(ain1),      /* Input data Ns[0] x Ns[1] */
+		             PyArray_STRIDES(ain1),   /* Input strides */
+		             PyArray_DATA(aout),      /* Output data */
+		             PyArray_STRIDES(aout),   /* Ouput strides */
+		             PyArray_DATA(ain2),      /* coefficients in filter */
+		             PyArray_STRIDES(ain2),   /* coefficients strides */ 
+		             PyArray_DIMS(ain2),      /* Size of kernel Nwin[2] */
+			     PyArray_DIMS(ain1),      /* Size of image Ns[0] x Ns[1] */
+		             flag,                    /* convolution parameters */
+		             PyArray_DATA(newfill));  /* fill value */
 
 
     switch (ret) {
@@ -1124,7 +1149,8 @@ fail:
 
 static char doc_order_filterND[] = "out = _order_filterND(a,domain,order)";
 
-static PyObject *sigtools_order_filterND(PyObject *NPY_UNUSED(dummy), PyObject *args) {
+static PyObject *sigtools_order_filterND(PyObject *NPY_UNUSED(dummy), 
+                                         PyObject *args) {
 	PyObject *domain, *a0;
 	int order=0;
 	
@@ -1148,18 +1174,19 @@ static PyObject *sigtools_remez(PyObject *NPY_UNUSED(dummy), PyObject *args) {
        
 
 
-        if (!PyArg_ParseTuple(args, "iOOO|idii", &numtaps, &bands, &des, &weight, &type, &Hz, &maxiter, &grid_density))
+        if (!PyArg_ParseTuple(args, "iOOO|idii", &numtaps, &bands, &des, &weight, 
+                              &type, &Hz, &maxiter, &grid_density))
 	    return NULL;
 
 	if (type != BANDPASS && type != DIFFERENTIATOR && type != HILBERT) {
-	  PyErr_SetString(PyExc_ValueError,
-			  "The type must be BANDPASS, DIFFERENTIATOR, or HILBERT.");
+	    PyErr_SetString(PyExc_ValueError,
+	                    "The type must be BANDPASS, DIFFERENTIATOR, or HILBERT.");
 	  return NULL;
 	}
 	
 	if (numtaps < 2) {
-	  PyErr_SetString(PyExc_ValueError,
-			  "The number of taps must be greater than 1.");
+	    PyErr_SetString(PyExc_ValueError,
+                            "The number of taps must be greater than 1.");
 	  return NULL;
 	}
 	
@@ -1172,17 +1199,20 @@ static PyObject *sigtools_remez(PyObject *NPY_UNUSED(dummy), PyObject *args) {
 	if (a_weight == NULL) goto fail;
 
 
-	numbands = a_des->dimensions[0];
-	if ((a_bands->dimensions[0] != 2*numbands) || (a_weight->dimensions[0] != numbands)) {
+	numbands = PyArray_DIMS(a_des)[0];
+	if ((PyArray_DIMS(a_bands)[0] != 2*numbands) || 
+            (PyArray_DIMS(a_weight)[0] != numbands)) {
 	  PyErr_SetString(PyExc_ValueError,
-			  "The inputs desired and weight must have same length.\n  The input bands must have twice this length.");
+			  "The inputs desired and weight must have same length.\n  " \
+                          "The input bands must have twice this length.");
 	  goto fail;
 	}
 
-      /* Check the bands input to see if it is monotonic, divide by 
-	 Hz to take from range 0 to 0.5 and check to see if in that range */ 
-
-	dptr = (double *)a_bands->data;
+      /* 
+       * Check the bands input to see if it is monotonic, divide by 
+       * Hz to take from range 0 to 0.5 and check to see if in that range
+       */ 
+	dptr = (double *)PyArray_DATA(a_bands);
 	oldvalue = 0;
 	for (k=0; k < 2*numbands; k++) {
 	  if (*dptr < oldvalue) {
@@ -1204,10 +1234,14 @@ static PyObject *sigtools_remez(PyObject *NPY_UNUSED(dummy), PyObject *args) {
 	h = (PyArrayObject *)PyArray_SimpleNew(1, &ret_dimens, NPY_DOUBLE);
 	if (h == NULL) goto fail;
 
-	err=pre_remez((double *)h->data, numtaps, numbands, (double *)a_bands->data, (double *)a_des->data, (double *)a_weight->data, type, maxiter, grid_density);
+	err=pre_remez((double *)PyArray_DATA(h), numtaps, numbands, 
+                      (double *)PyArray_DATA(a_bands),
+                      (double *)PyArray_DATA(a_des),
+                      (double *)PyArray_DATA(a_weight),
+                      type, maxiter, grid_density);
         if (err < 0) {
 	  if (err == -1) {
-            sprintf(mystr,"Failure to converge after %d iterations.\n      Design may still be correct.",maxiter);
+            sprintf(mystr, "Failure to converge after %d iterations.\n", maxiter);
 	    PyErr_SetString(PyExc_ValueError, mystr);
 	    goto fail;
 	  }
@@ -1254,13 +1288,13 @@ static PyObject *sigtools_median2d(PyObject *NPY_UNUSED(dummy), PyObject *args)
     if (size != NULL) {
 	a_size = (PyArrayObject *)PyArray_ContiguousFromObject(size, NPY_INTP, 1, 1);
 	if (a_size == NULL) goto fail;
-	if ((RANK(a_size) != 1) || (DIMS(a_size)[0] < 2)) 
+	if ((PyArray_NDIM(a_size) != 1) || (PyArray_DIMS(a_size)[0] < 2)) 
 	    PYERR("Size must be a length two sequence");
-	Nwin[0] = ((intp *)DATA(a_size))[0];
-	Nwin[1] = ((intp *)DATA(a_size))[1];
+	Nwin[0] = ((intp *)PyArray_DATA(a_size))[0];
+	Nwin[1] = ((intp *)PyArray_DATA(a_size))[1];
     }  
 
-    a_out = (PyArrayObject *)PyArray_SimpleNew(2,DIMS(a_image),typenum);
+    a_out = (PyArrayObject *)PyArray_SimpleNew(2, PyArray_DIMS(a_image), typenum);
     if (a_out == NULL) goto fail;
 
     if (setjmp(MALLOC_FAIL)) {
@@ -1269,13 +1303,19 @@ static PyObject *sigtools_median2d(PyObject *NPY_UNUSED(dummy), PyObject *args)
     else {
 	switch (typenum) {
 	case NPY_UBYTE:
-	    b_medfilt2((unsigned char *)DATA(a_image), (unsigned char *)DATA(a_out), Nwin, DIMS(a_image));
+	    b_medfilt2((unsigned char *)PyArray_DATA(a_image),
+                       (unsigned char *)PyArray_DATA(a_out),
+                       Nwin, PyArray_DIMS(a_image));
 	    break;
 	case NPY_FLOAT:
-	    f_medfilt2((float *)DATA(a_image), (float *)DATA(a_out), Nwin, DIMS(a_image));
+	    f_medfilt2((float *)PyArray_DATA(a_image),
+                       (float *)PyArray_DATA(a_out), Nwin,
+                       PyArray_DIMS(a_image));
 	    break;
 	case NPY_DOUBLE:
-	    d_medfilt2((double *)DATA(a_image), (double *)DATA(a_out), Nwin, DIMS(a_image));
+	    d_medfilt2((double *)PyArray_DATA(a_image),
+                       (double *)PyArray_DATA(a_out), Nwin,
+                       PyArray_DIMS(a_image));
 	    break;
 	default:
 	  PYERR("2D median filter only supports Int8, Float32, and Float64.");
