@@ -1,16 +1,16 @@
 """Base class for sparse matrices"""
 from __future__ import division, print_function, absolute_import
 
-__all__ = ['spmatrix', 'isspmatrix', 'issparse',
-        'SparseWarning','SparseEfficiencyWarning']
-
 import sys
-from warnings import warn
 
 import numpy as np
 
-from scipy.lib.six import xrange
-from .sputils import isdense, isscalarlike, isintlike
+from scipy._lib.six import xrange
+from .sputils import (isdense, isscalarlike, isintlike,
+                      get_sum_dtype, validateaxis)
+
+__all__ = ['spmatrix', 'isspmatrix', 'issparse',
+           'SparseWarning', 'SparseEfficiencyWarning']
 
 
 class SparseWarning(Warning):
@@ -26,29 +26,36 @@ class SparseEfficiencyWarning(SparseWarning):
 
 
 # The formats that we might potentially understand.
-_formats = {'csc':[0, "Compressed Sparse Column"],
-            'csr':[1, "Compressed Sparse Row"],
-            'dok':[2, "Dictionary Of Keys"],
-            'lil':[3, "LInked List"],
-            'dod':[4, "Dictionary of Dictionaries"],
-            'sss':[5, "Symmetric Sparse Skyline"],
-            'coo':[6, "COOrdinate"],
-            'lba':[7, "Linpack BAnded"],
-            'egd':[8, "Ellpack-itpack Generalized Diagonal"],
-            'dia':[9, "DIAgonal"],
-            'bsr':[10, "Block Sparse Row"],
-            'msr':[11, "Modified compressed Sparse Row"],
-            'bsc':[12, "Block Sparse Column"],
-            'msc':[13, "Modified compressed Sparse Column"],
-            'ssk':[14, "Symmetric SKyline"],
-            'nsk':[15, "Nonsymmetric SKyline"],
-            'jad':[16, "JAgged Diagonal"],
-            'uss':[17, "Unsymmetric Sparse Skyline"],
-            'vbr':[18, "Variable Block Row"],
-            'und':[19, "Undefined"]
+_formats = {'csc': [0, "Compressed Sparse Column"],
+            'csr': [1, "Compressed Sparse Row"],
+            'dok': [2, "Dictionary Of Keys"],
+            'lil': [3, "LInked List"],
+            'dod': [4, "Dictionary of Dictionaries"],
+            'sss': [5, "Symmetric Sparse Skyline"],
+            'coo': [6, "COOrdinate"],
+            'lba': [7, "Linpack BAnded"],
+            'egd': [8, "Ellpack-itpack Generalized Diagonal"],
+            'dia': [9, "DIAgonal"],
+            'bsr': [10, "Block Sparse Row"],
+            'msr': [11, "Modified compressed Sparse Row"],
+            'bsc': [12, "Block Sparse Column"],
+            'msc': [13, "Modified compressed Sparse Column"],
+            'ssk': [14, "Symmetric SKyline"],
+            'nsk': [15, "Nonsymmetric SKyline"],
+            'jad': [16, "JAgged Diagonal"],
+            'uss': [17, "Unsymmetric Sparse Skyline"],
+            'vbr': [18, "Variable Block Row"],
+            'und': [19, "Undefined"]
             }
 
 
+# These univariate ufuncs preserve zeros.
+_ufuncs_with_fixed_point_at_zero = frozenset([
+        np.sin, np.tan, np.arcsin, np.arctan, np.sinh, np.tanh, np.arcsinh,
+        np.arctanh, np.rint, np.sign, np.expm1, np.log1p, np.deg2rad,
+        np.rad2deg, np.floor, np.ceil, np.trunc, np.sqrt])
+
+
 MAXPRINT = 50
 
 
@@ -61,21 +68,20 @@ class spmatrix(object):
     ndim = 2
 
     def __init__(self, maxprint=MAXPRINT):
-        self.format = self.__class__.__name__[:3]
         self._shape = None
-        if self.format == 'spm':
+        if self.__class__.__name__ == 'spmatrix':
             raise ValueError("This class is not intended"
-                            " to be instantiated directly.")
+                             " to be instantiated directly.")
         self.maxprint = maxprint
 
-    def set_shape(self,shape):
+    def set_shape(self, shape):
         shape = tuple(shape)
 
         if len(shape) != 2:
-            raise ValueError("Only two-dimensional sparse arrays "
-                                     "are supported.")
+            raise ValueError("Only two-dimensional sparse "
+                             "arrays are supported.")
         try:
-            shape = int(shape[0]),int(shape[1])  # floats, other weirdness
+            shape = int(shape[0]), int(shape[1])  # floats, other weirdness
         except:
             raise TypeError('invalid shape')
 
@@ -95,8 +101,29 @@ class spmatrix(object):
 
     shape = property(fget=get_shape, fset=set_shape)
 
-    def reshape(self,shape):
-        raise NotImplementedError
+    def reshape(self, shape, order='C'):
+        """
+        Gives a new shape to a sparse matrix without changing its data.
+
+        Parameters
+        ----------
+        shape : length-2 tuple of ints
+            The new shape should be compatible with the original shape.
+        order : 'C', optional
+            This argument is in the signature *solely* for NumPy
+            compatibility reasons. Do not pass in anything except
+            for the default value, as this argument is not used.
+
+        Returns
+        -------
+        reshaped_matrix : `self` with the new dimensions of `shape`
+
+        See Also
+        --------
+        np.matrix.reshape : NumPy's implementation of 'reshape' for matrices
+        """
+        raise NotImplementedError("Reshaping not implemented for %s." %
+                                  self.__class__.__name__)
 
     def astype(self, t):
         return self.tocsr().astype(t).asformat(self.format)
@@ -104,7 +131,7 @@ class spmatrix(object):
     def asfptype(self):
         """Upcast matrix to a floating point format (if necessary)"""
 
-        fp_types = ['f','d','F','D']
+        fp_types = ['f', 'd', 'F', 'D']
 
         if self.dtype.char in fp_types:
             return self
@@ -114,58 +141,73 @@ class spmatrix(object):
                     return self.astype(fp_type)
 
             raise TypeError('cannot upcast [%s] to a floating '
-                             'point format' % self.dtype.name)
+                            'point format' % self.dtype.name)
 
     def __iter__(self):
         for r in xrange(self.shape[0]):
-            yield self[r,:]
+            yield self[r, :]
 
     def getmaxprint(self):
-        try:
-            maxprint = self.maxprint
-        except AttributeError:
-            maxprint = MAXPRINT
-        return maxprint
+        return self.maxprint
 
-    # def typecode(self):
-    #    try:
-    #        typ = self.dtype.char
-    #    except AttributeError:
-    #        typ = None
-    #    return typ
+    def count_nonzero(self):
+        """Number of non-zero entries, equivalent to
 
-    def getnnz(self):
-        try:
-            return self.nnz
-        except AttributeError:
-            raise AttributeError("nnz not defined")
+        np.count_nonzero(a.toarray())
+
+        Unlike getnnz() and the nnz property, which return the number of stored
+        entries (the length of the data attribute), this method counts the
+        actual number of non-zero entries in data.
+        """
+        raise NotImplementedError("count_nonzero not implemented for %s." %
+                                  self.__class__.__name__)
+
+    def getnnz(self, axis=None):
+        """Number of stored values, including explicit zeros.
+
+        Parameters
+        ----------
+        axis : None, 0, or 1
+            Select between the number of values across the whole matrix, in
+            each column, or in each row.
+
+        See also
+        --------
+        count_nonzero : Number of non-zero entries
+        """
+        raise NotImplementedError("getnnz not implemented for %s." %
+                                  self.__class__.__name__)
+
+    @property
+    def nnz(self):
+        """Number of stored values, including explicit zeros.
+
+        See also
+        --------
+        count_nonzero : Number of non-zero entries
+        """
+        return self.getnnz()
 
     def getformat(self):
-        try:
-            format = self.format
-        except AttributeError:
-            format = 'und'
-        return format
+        return getattr(self, 'format', 'und')
 
     def __repr__(self):
-        nnz = self.getnnz()
-        format = self.getformat()
+        _, format_name = _formats[self.getformat()]
         return "<%dx%d sparse matrix of type '%s'\n" \
                "\twith %d stored elements in %s format>" % \
-               (self.shape + (self.dtype.type, nnz, _formats[format][1]))
+               (self.shape + (self.dtype.type, self.nnz, format_name))
 
     def __str__(self):
         maxprint = self.getmaxprint()
 
         A = self.tocoo()
-        nnz = self.getnnz()
 
         # helper function, outputs "(i,j)  v"
-        def tostr(row,col,data):
-            triples = zip(list(zip(row,col)),data)
+        def tostr(row, col, data):
+            triples = zip(list(zip(row, col)), data)
             return '\n'.join([('  %s\t%s' % t) for t in triples])
 
-        if nnz > maxprint:
+        if self.nnz > maxprint:
             half = maxprint // 2
             out = tostr(A.row[:half], A.col[:half], A.data[:half])
             out += "\n  :\t:\n"
@@ -178,7 +220,7 @@ class spmatrix(object):
 
     def __bool__(self):  # Simple -- other ideas?
         if self.shape == (1, 1):
-            return True if self.nnz == 1 else False
+            return self.nnz != 0
         else:
             raise ValueError("The truth value of an array with more than one "
                              "element is ambiguous. Use a.any() or a.all().")
@@ -188,9 +230,8 @@ class spmatrix(object):
     # perhaps it should be the number of rows?  But for some uses the number of
     # non-zeros is more important.  For now, raise an exception!
     def __len__(self):
-        # return self.getnnz()
         raise TypeError("sparse matrix length is ambiguous; use getnnz()"
-                         " or shape[0]")
+                        " or shape[0]")
 
     def asformat(self, format):
         """Return this matrix in a given sparse format
@@ -210,7 +251,7 @@ class spmatrix(object):
         if format is None or format == self.format:
             return self
         else:
-            return getattr(self,'to' + format)()
+            return getattr(self, 'to' + format)()
 
     ###################################################################
     #  NOTE: All arithmetic operations use csr_matrix by default.
@@ -245,22 +286,25 @@ class spmatrix(object):
         """
         return self * other
 
+    def power(self, n, dtype=None):
+        return self.tocsr().power(n, dtype=dtype)
+
     def __eq__(self, other):
         return self.tocsr().__eq__(other)
 
     def __ne__(self, other):
         return self.tocsr().__ne__(other)
 
-    def __lt__(self,other):
+    def __lt__(self, other):
         return self.tocsr().__lt__(other)
 
-    def __gt__(self,other):
+    def __gt__(self, other):
         return self.tocsr().__gt__(other)
 
-    def __le__(self,other):
+    def __le__(self, other):
         return self.tocsr().__le__(other)
 
-    def __ge__(self,other):
+    def __ge__(self, other):
         return self.tocsr().__ge__(other)
 
     def __abs__(self):
@@ -288,7 +332,7 @@ class spmatrix(object):
         self._mul_sparse_matrix()
         """
 
-        M,N = self.shape
+        M, N = self.shape
 
         if other.__class__ is np.ndarray:
             # Fast path for the most common case
@@ -323,7 +367,7 @@ class spmatrix(object):
 
         if other.ndim == 1 or other.ndim == 2 and other.shape[1] == 1:
             # dense row or column vector
-            if other.shape != (N,) and other.shape != (N,1):
+            if other.shape != (N,) and other.shape != (N, 1):
                 raise ValueError('dimension mismatch')
 
             result = self._mul_vector(np.ravel(other))
@@ -333,7 +377,7 @@ class spmatrix(object):
 
             if other.ndim == 2 and other.shape[1] == 1:
                 # If 'other' was an (nx1) column vector, reshape the result
-                result = result.reshape(-1,1)
+                result = result.reshape(-1, 1)
 
             return result
 
@@ -377,6 +421,22 @@ class spmatrix(object):
                 tr = np.asarray(other).transpose()
             return (self.transpose() * tr).transpose()
 
+    #####################################
+    # matmul (@) operator (Python 3.5+) #
+    #####################################
+
+    def __matmul__(self, other):
+        if isscalarlike(other):
+            raise ValueError("Scalar operands are not allowed, "
+                             "use '*' instead")
+        return self.__mul__(other)
+
+    def __rmatmul__(self, other):
+        if isscalarlike(other):
+            raise ValueError("Scalar operands are not allowed, "
+                             "use '*' instead")
+        return self.__rmul__(other)
+
     ####################
     # Other Arithmetic #
     ####################
@@ -395,8 +455,8 @@ class spmatrix(object):
                 r = self._mul_scalar(1./other)
 
                 scalar_dtype = np.asarray(other).dtype
-                if (np.issubdtype(self.dtype, np.integer)
-                        and np.issubdtype(scalar_dtype, np.integer)):
+                if (np.issubdtype(self.dtype, np.integer) and
+                        np.issubdtype(scalar_dtype, np.integer)):
                     return r.astype(self.dtype)
                 else:
                     return r
@@ -443,19 +503,19 @@ class spmatrix(object):
         return -self.tocsr()
 
     def __iadd__(self, other):
-        raise NotImplementedError
+        return NotImplemented
 
     def __isub__(self, other):
-        raise NotImplementedError
+        return NotImplemented
 
     def __imul__(self, other):
-        raise NotImplementedError
+        return NotImplemented
 
     def __idiv__(self, other):
         return self.__itruediv__(other)
 
     def __itruediv__(self, other):
-        raise NotImplementedError
+        return NotImplemented
 
     def __pow__(self, other):
         if self.shape[0] != self.shape[1]:
@@ -480,7 +540,7 @@ class spmatrix(object):
         elif isscalarlike(other):
             raise ValueError('exponent must be an integer')
         else:
-            raise NotImplementedError
+            return NotImplemented
 
     def __getattr__(self, attr):
         if attr == 'A':
@@ -498,8 +558,32 @@ class spmatrix(object):
         else:
             raise AttributeError(attr + " not found")
 
-    def transpose(self):
-        return self.tocsr().transpose()
+    def transpose(self, axes=None, copy=False):
+        """
+        Reverses the dimensions of the sparse matrix.
+
+        Parameters
+        ----------
+        axes : None, optional
+            This argument is in the signature *solely* for NumPy
+            compatibility reasons. Do not pass in anything except
+            for the default value.
+        copy : bool, optional
+            Indicates whether or not attributes of `self` should be
+            copied whenever possible. The degree to which attributes
+            are copied varies depending on the type of sparse matrix
+            being used.
+
+        Returns
+        -------
+        p : `self` with the dimensions reversed.
+
+        See Also
+        --------
+        np.matrix.transpose : NumPy's implementation of 'transpose'
+                              for matrices
+        """
+        return self.tocsr().transpose(axes=axes, copy=copy)
 
     def conj(self):
         return self.tocsr().conj()
@@ -535,7 +619,7 @@ class spmatrix(object):
         # convert to COOrdinate format
         A = self.tocoo()
         nz_mask = A.data != 0
-        return (A.row[nz_mask],A.col[nz_mask])
+        return (A.row[nz_mask], A.col[nz_mask])
 
     def getcol(self, j):
         """Returns a copy of column j of the matrix, as an (m x 1) sparse
@@ -550,7 +634,8 @@ class spmatrix(object):
             j += n
         if j < 0 or j >= n:
             raise IndexError("index out of bounds")
-        col_selector = csc_matrix(([1], [[j], [0]]), shape=(n,1), dtype=self.dtype)
+        col_selector = csc_matrix(([1], [[j], [0]]),
+                                  shape=(n, 1), dtype=self.dtype)
         return self * col_selector
 
     def getrow(self, i):
@@ -566,7 +651,8 @@ class spmatrix(object):
             i += m
         if i < 0 or i >= m:
             raise IndexError("index out of bounds")
-        row_selector = csr_matrix(([1], [[0], [i]]), shape=(1,m), dtype=self.dtype)
+        row_selector = csr_matrix(([1], [[0], [i]]),
+                                  shape=(1, m), dtype=self.dtype)
         return row_selector * self
 
     # def __array__(self):
@@ -634,92 +720,222 @@ class spmatrix(object):
             returned after being modified in-place to contain the
             appropriate values.
         """
-        return self.tocoo().toarray(order=order, out=out)
+        return self.tocoo(copy=False).toarray(order=order, out=out)
 
-    def todok(self):
-        return self.tocoo().todok()
+    # Any sparse matrix format deriving from spmatrix must define one of
+    # tocsr or tocoo. The other conversion methods may be implemented for
+    # efficiency, but are not required.
+    def tocsr(self, copy=False):
+        """Convert this matrix to Compressed Sparse Row format.
 
-    def tocoo(self):
-        return self.tocsr().tocoo()
+        With copy=False, the data/indices may be shared between this matrix and
+        the resultant csr_matrix.
+        """
+        return self.tocoo(copy=copy).tocsr(copy=False)
 
-    def tolil(self):
-        return self.tocsr().tolil()
+    def todok(self, copy=False):
+        """Convert this matrix to Dictionary Of Keys format.
 
-    def todia(self):
-        return self.tocoo().todia()
+        With copy=False, the data/indices may be shared between this matrix and
+        the resultant dok_matrix.
+        """
+        return self.tocoo(copy=copy).todok(copy=False)
 
-    def tobsr(self, blocksize=None):
-        return self.tocsr().tobsr(blocksize=blocksize)
+    def tocoo(self, copy=False):
+        """Convert this matrix to COOrdinate format.
+
+        With copy=False, the data/indices may be shared between this matrix and
+        the resultant coo_matrix.
+        """
+        return self.tocsr(copy=False).tocoo(copy=copy)
+
+    def tolil(self, copy=False):
+        """Convert this matrix to LInked List format.
+
+        With copy=False, the data/indices may be shared between this matrix and
+        the resultant lil_matrix.
+        """
+        return self.tocsr(copy=False).tolil(copy=copy)
+
+    def todia(self, copy=False):
+        """Convert this matrix to sparse DIAgonal format.
+
+        With copy=False, the data/indices may be shared between this matrix and
+        the resultant dia_matrix.
+        """
+        return self.tocoo(copy=copy).todia(copy=False)
+
+    def tobsr(self, blocksize=None, copy=False):
+        """Convert this matrix to Block Sparse Row format.
+
+        With copy=False, the data/indices may be shared between this matrix and
+        the resultant bsr_matrix.
+
+        When blocksize=(R, C) is provided, it will be used for construction of
+        the bsr_matrix.
+        """
+        return self.tocsr(copy=False).tobsr(blocksize=blocksize, copy=copy)
+
+    def tocsc(self, copy=False):
+        """Convert this matrix to Compressed Sparse Column format.
+
+        With copy=False, the data/indices may be shared between this matrix and
+        the resultant csc_matrix.
+        """
+        return self.tocsr(copy=copy).tocsc(copy=False)
 
     def copy(self):
-        return self.__class__(self,copy=True)
+        """Returns a copy of this matrix.
 
-    def sum(self, axis=None):
-        """Sum the matrix over the given axis.  If the axis is None, sum
-        over both rows and columns, returning a scalar.
+        No data/indices will be shared between the returned value and current
+        matrix.
         """
-        # We use multiplication by an array of ones to achieve this.
+        return self.__class__(self, copy=True)
+
+    def sum(self, axis=None, dtype=None, out=None):
+        """
+        Sum the matrix elements over a given axis.
+
+        Parameters
+        ----------
+        axis : {-2, -1, 0, 1, None} optional
+            Axis along which the sum is computed. The default is to
+            compute the sum of all the matrix elements, returning a scalar
+            (i.e. `axis` = `None`).
+        dtype : dtype, optional
+            The type of the returned matrix and of the accumulator in which
+            the elements are summed.  The dtype of `a` is used by default
+            unless `a` has an integer dtype of less precision than the default
+            platform integer.  In that case, if `a` is signed then the platform
+            integer is used while if `a` is unsigned then an unsigned integer
+            of the same precision as the platform integer is used.
+
+            .. versionadded: 0.18.0
+
+        out : np.matrix, optional
+            Alternative output matrix in which to place the result. It must
+            have the same shape as the expected output, but the type of the
+            output values will be cast if necessary.
+
+            .. versionadded: 0.18.0
+
+        Returns
+        -------
+        sum_along_axis : np.matrix
+            A matrix with the same shape as `self`, with the specified
+            axis removed.
+
+        See Also
+        --------
+        np.matrix.sum : NumPy's implementation of 'sum' for matrices
+
+        """
+        validateaxis(axis)
+
+        # We use multiplication by a matrix of ones to achieve this.
         # For some sparse matrix formats more efficient methods are
         # possible -- these should override this function.
         m, n = self.shape
 
         # Mimic numpy's casting.
-        if np.issubdtype(self.dtype, np.float_):
-            res_dtype = np.float_
-        elif (np.issubdtype(self.dtype, np.int_) or
-              np.issubdtype(self.dtype, np.bool_)):
-                res_dtype = np.int_
-        elif np.issubdtype(self.dtype, np.complex_):
-            res_dtype = np.complex_
-        else:
-            res_dtype = self.dtype
+        res_dtype = get_sum_dtype(self.dtype)
 
         if axis is None:
             # sum over rows and columns
-            return (self * np.asmatrix(np.ones((n, 1), dtype=res_dtype))).sum()
+            return (self * np.asmatrix(np.ones(
+                (n, 1), dtype=res_dtype))).sum(
+                dtype=dtype, out=out)
 
         if axis < 0:
             axis += 2
+
+        # axis = 0 or 1 now
         if axis == 0:
             # sum over columns
-            return np.asmatrix(np.ones((1, m), dtype=res_dtype)) * self
-        elif axis == 1:
+            ret = np.asmatrix(np.ones(
+                (1, m), dtype=res_dtype)) * self
+        else:
             # sum over rows
-            return self * np.asmatrix(np.ones((n, 1), dtype=res_dtype))
-        else:
-            raise ValueError("axis out of bounds")
+            ret = self * np.asmatrix(
+                np.ones((n, 1), dtype=res_dtype))
 
-    def mean(self, axis=None):
-        """Average the matrix over the given axis.  If the axis is None,
-        average over both rows and columns, returning a scalar.
+        if out is not None and out.shape != ret.shape:
+            raise ValueError("dimensions do not match")
+
+        return ret.sum(axis=(), dtype=dtype, out=out)
+
+    def mean(self, axis=None, dtype=None, out=None):
         """
-        # Mimic numpy's casting.  The int32/int64 check works around numpy
-        # 1.5.x behavior of np.issubdtype, see gh-2677.
-        if (np.issubdtype(self.dtype, np.float_) or
-            np.issubdtype(self.dtype, np.int_) or
-            self.dtype in [np.dtype('int32'), np.dtype('int64')] or
-            np.issubdtype(self.dtype, np.bool_)):
-                res_dtype = np.float_
-        elif np.issubdtype(self.dtype, np.complex_):
-            res_dtype = np.complex_
+        Compute the arithmetic mean along the specified axis.
+
+        Returns the average of the matrix elements. The average is taken
+        over all elements in the matrix by default, otherwise over the
+        specified axis. `float64` intermediate and return values are used
+        for integer inputs.
+
+        Parameters
+        ----------
+        axis : {-2, -1, 0, 1, None} optional
+            Axis along which the mean is computed. The default is to compute
+            the mean of all elements in the matrix (i.e. `axis` = `None`).
+        dtype : data-type, optional
+            Type to use in computing the mean. For integer inputs, the default
+            is `float64`; for floating point inputs, it is the same as the
+            input dtype.
+
+            .. versionadded: 0.18.0
+
+        out : np.matrix, optional
+            Alternative output matrix in which to place the result. It must
+            have the same shape as the expected output, but the type of the
+            output values will be cast if necessary.
+
+            .. versionadded: 0.18.0
+
+        Returns
+        -------
+        m : np.matrix
+
+        See Also
+        --------
+        np.matrix.mean : NumPy's implementation of 'mean' for matrices
+
+        """
+        def _is_integral(dtype):
+            return (np.issubdtype(dtype, np.integer) or
+                    np.issubdtype(dtype, np.bool_))
+
+        validateaxis(axis)
+
+        res_dtype = self.dtype.type
+        integral = _is_integral(self.dtype)
+
+        # output dtype
+        if dtype is None:
+            if integral:
+                res_dtype = np.float64
         else:
-            res_dtype = self.dtype
+            res_dtype = np.dtype(dtype).type
+
+        # intermediate dtype for summation
+        inter_dtype = np.float64 if integral else res_dtype
+        inter_self = self.astype(inter_dtype)
 
         if axis is None:
-            return self.sum(None) * 1.0 / (self.shape[0]*self.shape[1])
+            return (inter_self / np.array(
+                self.shape[0] * self.shape[1]))\
+                .sum(dtype=res_dtype, out=out)
 
         if axis < 0:
             axis += 2
+
+        # axis = 0 or 1 now
         if axis == 0:
-            mean = self.astype(res_dtype).sum(0)
-            mean *= 1.0 / self.shape[0]
-            return mean
-        elif axis == 1:
-            mean = self.astype(res_dtype).sum(1)
-            mean *= 1.0 / self.shape[1]
-            return mean
+            return (inter_self * (1.0 / self.shape[0])).sum(
+                axis=0, dtype=res_dtype, out=out)
         else:
-            raise ValueError("axis out of bounds")
+            return (inter_self * (1.0 / self.shape[1])).sum(
+                axis=1, dtype=res_dtype, out=out)
 
     def diagonal(self):
         """Returns the main diagonal of the matrix
@@ -750,8 +966,12 @@ class spmatrix(object):
         M, N = self.shape
         if (k > 0 and k >= N) or (k < 0 and -k >= M):
             raise ValueError("k exceeds matrix dimensions")
+        self._setdiag(np.asarray(values), k)
+
+    def _setdiag(self, values, k):
+        M, N = self.shape
         if k < 0:
-            if np.asarray(values).ndim == 0:
+            if values.ndim == 0:
                 # broadcast
                 max_index = min(M+k, N)
                 for i in xrange(max_index):
@@ -760,10 +980,10 @@ class spmatrix(object):
                 max_index = min(M+k, N, len(values))
                 if max_index <= 0:
                     return
-                for i,v in enumerate(values[:max_index]):
+                for i, v in enumerate(values[:max_index]):
                     self[i - k, i] = v
         else:
-            if np.asarray(values).ndim == 0:
+            if values.ndim == 0:
                 # broadcast
                 max_index = min(M, N-k)
                 for i in xrange(max_index):
@@ -772,7 +992,7 @@ class spmatrix(object):
                 max_index = min(M, N-k, len(values))
                 if max_index <= 0:
                     return
-                for i,v in enumerate(values[:max_index]):
+                for i, v in enumerate(values[:max_index]):
                     self[i, i + k] = v
 
     def _process_toarray_args(self, order, out):
@@ -830,16 +1050,16 @@ class spmatrix(object):
                                   rdivide=rdivide)
         elif func is np.true_divide:
             rdivide = (pos == 1)
-            result = self._divide(*without_self, true_divide=True, rdivide=rdivide)
+            result = self._divide(*without_self,
+                                  true_divide=True,
+                                  rdivide=rdivide)
         elif func is np.maximum:
             result = self.maximum(*without_self)
         elif func is np.minimum:
             result = self.minimum(*without_self)
         elif func is np.absolute:
             result = abs(self)
-        elif func in (np.sin, np.tan, np.arcsin, np.arctan, np.sinh, np.tanh,
-                      np.arcsinh, np.arctanh, np.rint, np.sign, np.expm1, np.log1p,
-                      np.deg2rad, np.rad2deg, np.floor, np.ceil, np.trunc, np.sqrt):
+        elif func in _ufuncs_with_fixed_point_at_zero:
             func_name = func.__name__
             if hasattr(self, func_name):
                 result = getattr(self, func_name)()
