@@ -7,15 +7,15 @@ from warnings import warn
 import operator
 
 import numpy as np
-from scipy.lib.six import xrange, zip as izip
+from scipy._lib.six import zip as izip
 
 from .base import spmatrix, isspmatrix, SparseEfficiencyWarning
 from .data import _data_matrix, _minmax_mixin
 from .dia import dia_matrix
 from . import _sparsetools
-from .sputils import upcast, upcast_char, to_native, isdense, isshape, \
-     getdtype, isscalarlike, isintlike, IndexMixin, get_index_dtype, \
-     downcast_intp_index, _compat_unique, _compat_bincount
+from .sputils import (upcast, upcast_char, to_native, isdense, isshape,
+                      getdtype, isscalarlike, IndexMixin, get_index_dtype,
+                      downcast_intp_index, get_sum_dtype)
 
 
 class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
@@ -37,7 +37,9 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
                 # create empty matrix
                 self.shape = arg1   # spmatrix checks for errors here
                 M, N = self.shape
-                idx_dtype = get_index_dtype(maxval=self._swap((M,N))[1])
+                # Select index dtype large enough to pass array and
+                # scalar parameters to sparsetools
+                idx_dtype = get_index_dtype(maxval=max(M,N))
                 self.data = np.zeros(0, getdtype(dtype, default=float))
                 self.indices = np.zeros(0, idx_dtype)
                 self.indptr = np.zeros(self._swap((M,N))[0] + 1, dtype=idx_dtype)
@@ -50,10 +52,17 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
                 elif len(arg1) == 3:
                     # (data, indices, indptr) format
                     (data, indices, indptr) = arg1
-                    idx_dtype = get_index_dtype((indices, indptr), check_contents=True)
+
+                    # Select index dtype large enough to pass array and
+                    # scalar parameters to sparsetools
+                    maxval = None
+                    if shape is not None:
+                        maxval = max(shape)
+                    idx_dtype = get_index_dtype((indices, indptr), maxval=maxval, check_contents=True)
+
                     self.indices = np.array(indices, copy=copy, dtype=idx_dtype)
                     self.indptr = np.array(indptr, copy=copy, dtype=idx_dtype)
-                    self.data = np.array(data, copy=copy, dtype=getdtype(dtype, data))
+                    self.data = np.array(data, copy=copy, dtype=dtype)
                 else:
                     raise ValueError("unrecognized %s_matrix constructor usage" %
                             self.format)
@@ -88,14 +97,6 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
         self.check_format(full_check=False)
 
     def getnnz(self, axis=None):
-        """Get the count of explicitly-stored values (nonzeros)
-
-        Parameters
-        ----------
-        axis : None, 0, or 1
-            Select between the number of values across the whole matrix, in
-            each column, or in each row.
-        """
         if axis is None:
             return int(self.indptr[-1])
         else:
@@ -104,13 +105,13 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
             axis, _ = self._swap((axis, 1 - axis))
             _, N = self._swap(self.shape)
             if axis == 0:
-                return _compat_bincount(downcast_intp_index(self.indices),
-                                        minlength=N)
+                return np.bincount(downcast_intp_index(self.indices),
+                                   minlength=N)
             elif axis == 1:
                 return np.diff(self.indptr)
             raise ValueError('axis out of bounds')
 
-    nnz = property(fget=getnnz)
+    getnnz.__doc__ = spmatrix.getnnz.__doc__
 
     def _set_self(self, other, copy=False):
         """take the member variables of other and assign them to self"""
@@ -127,12 +128,10 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
         """check whether the matrix format is valid
 
         Parameters
-        ==========
-
-            - full_check : {bool}
-                - True  - rigorous check, O(N) operations : default
-                - False - basic check, O(1) operations
-
+        ----------
+        full_check : bool, optional
+            If `True`, rigorous check, O(N) operations. Otherwise
+            basic check, O(1) operations (default True).
         """
         # use _swap to determine proper bounds
         major_name,minor_name = self._swap(('row','column'))
@@ -194,21 +193,14 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
     # Boolean comparisons #
     #######################
 
-    def _copy_with_const(self, const):
-        """Copy data, with all nonzeros replaced with constant for binopt
-
-        Adopts the dtype of const to avoid removing sign or magnitude before
-        comparison.
-
-        Warning: does not make a copy of indices and indptr
+    def _scalar_binopt(self, other, op):
+        """Scalar version of self._binopt, for cases in which no new nonzeros
+        are added. Produces a new spmatrix in canonical form.
         """
-        try:
-            self.sum_duplicates()
-        except NotImplementedError:
-            pass
-        data = np.empty(self.data.shape, dtype=np.asarray(const).dtype)
-        data.fill(const)
-        return self.__class__((data, self.indices, self.indptr), shape=self.shape)
+        self.sum_duplicates()
+        res = self._with_data(op(self.data, other), copy=True)
+        res.eliminate_zeros()
+        return res
 
     def __eq__(self, other):
         # Scalar other.
@@ -216,16 +208,14 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
             if np.isnan(other):
                 return self.__class__(self.shape, dtype=np.bool_)
 
-            other_arr = self._copy_with_const(other)
-            res = self._binopt(other_arr,'_ne_')
             if other == 0:
                 warn("Comparing a sparse matrix with 0 using == is inefficient"
                         ", try using != instead.", SparseEfficiencyWarning)
                 all_true = self.__class__(np.ones(self.shape, dtype=np.bool_))
-                return all_true - res
+                inv = self._scalar_binopt(other, operator.ne)
+                return all_true - inv
             else:
-                sparsity_pattern = self._copy_with_const(True)
-                return sparsity_pattern - res
+                return self._scalar_binopt(other, operator.eq)
         # Dense other.
         elif isdense(other):
             return self.todense() == other
@@ -256,11 +246,10 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
                 warn("Comparing a sparse matrix with a nonzero scalar using !="
                      " is inefficient, try using == instead.", SparseEfficiencyWarning)
                 all_true = self.__class__(np.ones(self.shape), dtype=np.bool_)
-                res = (self == other)
-                return all_true - res
+                inv = self._scalar_binopt(other, operator.eq)
+                return all_true - inv
             else:
-                other_arr = self._copy_with_const(other)
-                return self._binopt(other_arr,'_ne_')
+                return self._scalar_binopt(other, operator.ne)
         # Dense other.
         elif isdense(other):
             return self.todense() != other
@@ -282,13 +271,12 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
                 raise NotImplementedError(" >= and <= don't work with 0.")
             elif op(0, other):
                 warn(bad_scalar_msg, SparseEfficiencyWarning)
-                other_arr = np.empty(self.shape, dtype=np.asarray(other).dtype)
+                other_arr = np.empty(self.shape, dtype=np.result_type(other))
                 other_arr.fill(other)
                 other_arr = self.__class__(other_arr)
                 return self._binopt(other_arr, op_name)
             else:
-                other_arr = self._copy_with_const(other)
-                return self._binopt(other_arr, op_name)
+                return self._scalar_binopt(other, op)
         # Dense other.
         elif isdense(other):
             return op(self.todense(), other)
@@ -327,6 +315,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
                                 "Comparing a sparse matrix with a scalar "
                                 "greater than zero using <= is inefficient, "
                                 "try using > instead.")
+
     def __ge__(self,other):
         return self._inequality(other, operator.ge, '_ge_',
                                 "Comparing a sparse matrix with a scalar "
@@ -354,7 +343,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
             # Convert this matrix to a dense matrix and add them
             return self.todense() + other
         else:
-            raise NotImplementedError
+            return NotImplemented
 
     def __radd__(self,other):
         return self.__add__(other)
@@ -376,7 +365,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
             # Convert this matrix to a dense matrix and subtract them
             return self.todense() - other
         else:
-            raise NotImplementedError
+            return NotImplemented
 
     def __rsub__(self,other):  # other - self
         # note: this can't be replaced by other + (-self) for unsigned types
@@ -390,7 +379,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
             # Convert this matrix to a dense matrix and subtract them
             return other - self.todense()
         else:
-            raise NotImplementedError
+            return NotImplemented
 
     def multiply(self, other):
         """Point-wise multiplication by another matrix, vector, or
@@ -398,7 +387,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
         """
         # Scalar multiplication.
         if isscalarlike(other):
-            return self.__mul__(other)
+            return self._mul_scalar(other)
         # Sparse matrix or vector.
         if isspmatrix(other):
             if self.shape == other.shape:
@@ -406,9 +395,9 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
                 return self._binopt(other, '_elmul_')
             # Single element.
             elif other.shape == (1,1):
-                return self.__mul__(other.tocsc().data[0])
+                return self._mul_scalar(other.toarray()[0, 0])
             elif self.shape == (1,1):
-                return other.__mul__(self.tocsc().data[0])
+                return other._mul_scalar(self.toarray()[0, 0])
             # A row times a column.
             elif self.shape[1] == other.shape[0] and self.shape[1] == 1:
                 return self._mul_sparse_matrix(other.tocsc())
@@ -436,6 +425,16 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
                 return copy._mul_sparse_matrix(other)
             else:
                 raise ValueError("inconsistent shapes")
+        # Dense matrix.
+        if isdense(other):
+            if self.shape == other.shape:
+                ret = self.tocoo()
+                ret.data = np.multiply(ret.data, other[ret.row, ret.col]
+                                       ).view(np.ndarray).ravel()
+                return ret
+            # Single element.
+            elif other.size == 1:
+                return self._mul_scalar(other.flat[0])
         # Anything else.
         return np.multiply(self.todense(), other)
 
@@ -474,7 +473,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
         K2, N = other.shape
 
         major_axis = self._swap((M,N))[0]
-        other = self.__class__(other) # convert to this format
+        other = self.__class__(other)  # convert to this format
 
         idx_dtype = get_index_dtype((self.indptr, self.indices,
                                      other.indptr, other.indices),
@@ -532,10 +531,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
                 other_arr = self.__class__(other_arr)
                 return self._binopt(other_arr, op_name)
             else:
-                try:
-                    self.sum_duplicates()
-                except NotImplementedError:
-                    pass
+                self.sum_duplicates()
                 new_data = npop(self.data, np.asarray(other))
                 mat = self.__class__((new_data, self.indices, self.indptr),
                                      dtype=new_data.dtype, shape=self.shape)
@@ -557,29 +553,34 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
     # Reduce operations #
     #####################
 
-    def sum(self, axis=None):
+    def sum(self, axis=None, dtype=None, out=None):
         """Sum the matrix over the given axis.  If the axis is None, sum
         over both rows and columns, returning a scalar.
         """
         # The spmatrix base class already does axis=0 and axis=1 efficiently
         # so we only do the case axis=None here
-        if axis is None:
-            return self.data.sum()
-        elif (not hasattr(self, 'blocksize') and
-              axis in self._swap(((1, -1), (0, 2)))[0]):
+        if (not hasattr(self, 'blocksize') and
+                    axis in self._swap(((1, -1), (0, 2)))[0]):
             # faster than multiplication for large minor axis in CSC/CSR
-            dtype = self.dtype
-            if np.issubdtype(dtype, np.bool_):
-                dtype = np.int_
-            ret = np.zeros(len(self.indptr) - 1, dtype=dtype)
+            res_dtype = get_sum_dtype(self.dtype)
+            ret = np.zeros(len(self.indptr) - 1, dtype=res_dtype)
+
             major_index, value = self._minor_reduce(np.add)
             ret[major_index] = value
             ret = np.asmatrix(ret)
             if axis % 2 == 1:
                 ret = ret.T
-            return ret
+
+            if out is not None and out.shape != ret.shape:
+                raise ValueError('dimensions do not match')
+
+            return ret.sum(axis=(), dtype=dtype, out=out)
+        # spmatrix will handle the remaining situations when axis
+        # is in {None, -1, 0, 1}
         else:
-            return spmatrix.sum(self, axis)
+            return spmatrix.sum(self, axis=axis, dtype=dtype, out=out)
+
+    sum.__doc__ = spmatrix.sum.__doc__
 
     def _minor_reduce(self, ufunc):
         """Reduce nonzeros with a ufunc over the minor axis when non-empty
@@ -607,59 +608,82 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
     # Getting and Setting #
     #######################
 
-    def __getitem__(self, key):
-        if isinstance(key, tuple):
-            row = key[0]
-            col = key[1]
-
-            # TODO implement CSR[ [1,2,3], X ] with sparse matmat
-            # TODO make use of sorted indices
-
-            if isintlike(row) and isintlike(col):
-                return self._get_single_element(row,col)
-            else:
-                major,minor = self._swap((row,col))
-                if isintlike(major) and isinstance(minor,slice):
-                    minor_shape = self._swap(self.shape)[1]
-                    start, stop, stride = minor.indices(minor_shape)
-                    out_shape = self._swap((1, stop-start))
-                    return self._get_slice(major, start, stop, stride, out_shape)
-                elif isinstance(row, slice) or isinstance(col, slice):
-                    return self._get_submatrix(row, col)
-                else:
-                    raise NotImplementedError
-
-        elif isintlike(key):
-            return self[key, :]
-        else:
-            raise IndexError("invalid index")
-
     def __setitem__(self, index, x):
         # Process arrays from IndexMixin
         i, j = self._unpack_index(index)
         i, j = self._index_to_arrays(i, j)
 
         if isspmatrix(x):
-            x = x.toarray()
+            broadcast_row = x.shape[0] == 1 and i.shape[0] != 1
+            broadcast_col = x.shape[1] == 1 and i.shape[1] != 1
+            if not ((broadcast_row or x.shape[0] == i.shape[0]) and
+                    (broadcast_col or x.shape[1] == i.shape[1])):
+                raise ValueError("shape mismatch in assignment")
 
-        # Make x and i into the same shape
-        x = np.asarray(x, dtype=self.dtype)
-        x, _ = np.broadcast_arrays(x, i)
+            # clear entries that will be overwritten
+            ci, cj = self._swap((i.ravel(), j.ravel()))
+            self._zero_many(ci, cj)
 
-        if x.shape != i.shape:
-            raise ValueError("shape mismatch in assignment")
+            x = x.tocoo()
+            r, c = x.row, x.col
+            x = np.asarray(x.data, dtype=self.dtype)
+            if broadcast_row:
+                r = np.repeat(np.arange(i.shape[0]), len(r))
+                c = np.tile(c, i.shape[0])
+                x = np.tile(x, i.shape[0])
+            if broadcast_col:
+                r = np.repeat(r, i.shape[1])
+                c = np.tile(np.arange(i.shape[1]), len(c))
+                x = np.repeat(x, i.shape[1])
+            # only assign entries in the new sparsity structure
+            i = i[r, c]
+            j = j[r, c]
+        else:
+            # Make x and i into the same shape
+            x = np.asarray(x, dtype=self.dtype)
+            x, _ = np.broadcast_arrays(x, i)
+
+            if x.shape != i.shape:
+                raise ValueError("shape mismatch in assignment")
 
         if np.size(x) == 0:
             return
         i, j = self._swap((i.ravel(), j.ravel()))
         self._set_many(i, j, x.ravel())
 
-    def _set_many(self, i, j, x):
-        """Sets value at each (i, j) to x
+    def _setdiag(self, values, k):
+        if 0 in self.shape:
+            return
 
-        Here (i,j) index major and minor respectively.
-        """
+        M, N = self.shape
+        broadcast = (values.ndim == 0)
+
+        if k < 0:
+            if broadcast:
+                max_index = min(M + k, N)
+            else:
+                max_index = min(M + k, N, len(values))
+            i = np.arange(max_index, dtype=self.indices.dtype)
+            j = np.arange(max_index, dtype=self.indices.dtype)
+            i -= k
+
+        else:
+            if broadcast:
+                max_index = min(M, N - k)
+            else:
+                max_index = min(M, N - k, len(values))
+            i = np.arange(max_index, dtype=self.indices.dtype)
+            j = np.arange(max_index, dtype=self.indices.dtype)
+            j += k
+
+        if not broadcast:
+            values = values[:len(i)]
+
+        self[i, j] = values
+
+    def _prepare_indices(self, i, j):
         M, N = self._swap(self.shape)
+
         def check_bounds(indices, bound):
             idx = indices.max()
             if idx >= bound:
@@ -675,6 +699,14 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
 
         i = np.asarray(i, dtype=self.indices.dtype)
         j = np.asarray(j, dtype=self.indices.dtype)
+        return i, j, M, N
+
+    def _set_many(self, i, j, x):
+        """Sets value at each (i, j) to x
+
+        Here (i,j) index major and minor respectively.
+        """
+        i, j, M, N = self._prepare_indices(i, j)
 
         n_samples = len(x)
         offsets = np.empty(n_samples, dtype=self.indices.dtype)
@@ -707,6 +739,27 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
             j[j < 0] += N
             self._insert_many(i, j, x[mask])
 
+    def _zero_many(self, i, j):
+        """Sets value at each (i, j) to zero, preserving sparsity structure.
+
+        Here (i,j) index major and minor respectively.
+        """
+        i, j, M, N = self._prepare_indices(i, j)
+
+        n_samples = len(i)
+        offsets = np.empty(n_samples, dtype=self.indices.dtype)
+        ret = _sparsetools.csr_sample_offsets(M, N, self.indptr, self.indices,
+                                              n_samples, i, j, offsets)
+        if ret == 1:
+            # rinse and repeat
+            self.sum_duplicates()
+            _sparsetools.csr_sample_offsets(M, N, self.indptr,
+                                            self.indices, n_samples, i, j,
+                                            offsets)
+
+        # only assign zeros to the existing sparsity structure
+        self.data[offsets[offsets > -1]] = 0
+
     def _insert_many(self, i, j, x):
         """Inserts new nonzero at each (i, j) with value x
 
@@ -734,7 +787,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
         # Collate old and new in chunks by major index
         indices_parts = []
         data_parts = []
-        ui, ui_indptr = _compat_unique(i, return_index=True)
+        ui, ui_indptr = np.unique(i, return_index=True)
         ui_indptr = np.append(ui_indptr, len(j))
         new_nnzs = np.diff(ui_indptr)
         prev = 0
@@ -746,7 +799,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
             data_parts.append(self.data[start:stop])
 
             # handle duplicate j: keep last setting
-            uj, uj_indptr = _compat_unique(j[js:je][::-1], return_index=True)
+            uj, uj_indptr = np.unique(j[js:je][::-1], return_index=True)
             if len(uj) == je - js:
                 indices_parts.append(j[js:je])
                 data_parts.append(x[js:je])
@@ -771,6 +824,7 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
 
         if do_sort:
             # TODO: only sort where necessary
+            self.has_sorted_indices = False
             self.sort_indices()
 
         self.check_format(full_check=False)
@@ -794,31 +848,6 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
         return np.compress(minor_index == self.indices[start:end],
                            self.data[start:end]).sum(dtype=self.dtype)
 
-    def _get_slice(self, i, start, stop, stride, shape):
-        """Returns a copy of the elements
-            [i, start:stop:string] for row-oriented matrices
-            [start:stop:string, i] for column-oriented matrices
-        """
-        if stride != 1:
-            raise ValueError("slicing with step != 1 not supported")
-        if stop <= start:
-            raise ValueError("slice width must be >= 1")
-
-        # TODO make [i,:] faster
-        # TODO implement [i,x:y:z]
-
-        indices = []
-
-        for ind in xrange(self.indptr[i], self.indptr[i+1]):
-            if self.indices[ind] >= start and self.indices[ind] < stop:
-                indices.append(ind)
-
-        index = self.indices[indices] - start
-        data = self.data[indices]
-        indptr = np.array([0, len(indices)])
-        return self.__class__((data, index, indptr), shape=shape,
-                              dtype=self.dtype)
-
     def _get_submatrix(self, slice0, slice1):
         """Return a submatrix of this matrix (new matrix is created)."""
 
@@ -873,34 +902,18 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
     # Conversion methods #
     ######################
 
-    def todia(self):
-        return self.tocoo(copy=False).todia()
-
-    def todok(self):
-        return self.tocoo(copy=False).todok()
-
-    def tocoo(self,copy=True):
-        """Return a COOrdinate representation of this matrix
-
-        When copy=False the index and data arrays are not copied.
-        """
-        major_dim,minor_dim = self._swap(self.shape)
-
-        data = self.data
+    def tocoo(self, copy=True):
+        major_dim, minor_dim = self._swap(self.shape)
         minor_indices = self.indices
-
-        if copy:
-            data = data.copy()
-            minor_indices = minor_indices.copy()
-
         major_indices = np.empty(len(minor_indices), dtype=self.indices.dtype)
-
-        _sparsetools.expandptr(major_dim,self.indptr,major_indices)
-
-        row,col = self._swap((major_indices,minor_indices))
+        _sparsetools.expandptr(major_dim, self.indptr, major_indices)
+        row, col = self._swap((major_indices, minor_indices))
 
         from .coo import coo_matrix
-        return coo_matrix((data,(row,col)), self.shape)
+        return coo_matrix((self.data, (row, col)), self.shape, copy=copy,
+                          dtype=self.dtype)
+
+    tocoo.__doc__ = spmatrix.tocoo.__doc__
 
     def toarray(self, order=None, out=None):
         """See the docstring for `spmatrix.toarray`."""
@@ -913,12 +926,11 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
     def eliminate_zeros(self):
         """Remove zero entries from the matrix
 
-        The is an *in place* operation
+        This is an *in place* operation
         """
-        fn = _sparsetools.csr_eliminate_zeros
-        M,N = self._swap(self.shape)
-        fn(M, N, self.indptr, self.indices, self.data)
-
+        M, N = self._swap(self.shape)
+        _sparsetools.csr_eliminate_zeros(M, N, self.indptr, self.indices,
+                                         self.data)
         self.prune()  # nnz may have changed
 
     def __get_has_canonical_format(self):
@@ -938,9 +950,8 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
             # not sorted => not canonical
             self._has_canonical_format = False
         elif not hasattr(self, '_has_canonical_format'):
-            fn = _sparsetools.csr_has_canonical_format
-            self.has_canonical_format = \
-                    fn(len(self.indptr) - 1, self.indptr, self.indices)
+            self.has_canonical_format = _sparsetools.csr_has_canonical_format(
+                len(self.indptr) - 1, self.indptr, self.indices)
         return self._has_canonical_format
 
     def __set_has_canonical_format(self, val):
@@ -960,9 +971,9 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
             return
         self.sort_indices()
 
-        fn = _sparsetools.csr_sum_duplicates
-        M,N = self._swap(self.shape)
-        fn(M, N, self.indptr, self.indices, self.data)
+        M, N = self._swap(self.shape)
+        _sparsetools.csr_sum_duplicates(M, N, self.indptr, self.indices,
+                                        self.data)
 
         self.prune()  # nnz may have changed
         self.has_canonical_format = True
@@ -978,9 +989,8 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
 
         # first check to see if result was cached
         if not hasattr(self,'_has_sorted_indices'):
-            fn = _sparsetools.csr_has_sorted_indices
-            self._has_sorted_indices = \
-                    fn(len(self.indptr) - 1, self.indptr, self.indices)
+            self._has_sorted_indices = _sparsetools.csr_has_sorted_indices(
+                len(self.indptr) - 1, self.indptr, self.indices)
         return self._has_sorted_indices
 
     def __set_sorted(self, val):
@@ -1004,8 +1014,8 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
         """
 
         if not self.has_sorted_indices:
-            fn = _sparsetools.csr_sort_indices
-            fn(len(self.indptr) - 1, self.indptr, self.indices, self.data)
+            _sparsetools.csr_sort_indices(len(self.indptr) - 1, self.indptr,
+                                          self.indices, self.data)
             self.has_sorted_indices = True
 
     def prune(self):
@@ -1092,10 +1102,13 @@ class _cs_matrix(_data_matrix, _minmax_mixin, IndexMixin):
 
         if np.issubdtype(r.dtype, np.inexact):
             # Eldiv leaves entries outside the combined sparsity
-            # pattern empty, so they must be filled manually. They are
-            # always nan, so that the matrix is completely full.
+            # pattern empty, so they must be filled manually.
+            # Everything outside of other's sparsity is NaN, and everything
+            # inside it is either zero or defined by eldiv.
             out = np.empty(self.shape, dtype=self.dtype)
             out.fill(np.nan)
+            row, col = other.nonzero()
+            out[row, col] = 0
             r = r.tocoo()
             out[r.row, r.col] = r.data
             out = np.matrix(out)
