@@ -64,7 +64,7 @@ int ilu_zdrop_row(
     int m, n; /* m x n is the size of the supernode */
     int r = 0; /* number of dropped rows */
     register double *temp;
-    register doublecomplex *lusup = Glu->lusup;
+    register doublecomplex *lusup = (doublecomplex *) Glu->lusup;
     register int *lsub = Glu->lsub;
     register int *xlsub = Glu->xlsub;
     register int *xlusup = Glu->xlusup;
