@@ -52,7 +52,7 @@ zcopy_to_ucol(
     supno   = Glu->supno;
     lsub    = Glu->lsub;
     xlsub   = Glu->xlsub;
-    ucol    = Glu->ucol;
+    ucol    = (doublecomplex *) Glu->ucol;
     usub    = Glu->usub;
     xusub   = Glu->xusub;
     nzumax  = Glu->nzumax;
@@ -76,7 +76,7 @@ zcopy_to_ucol(
 		while ( new_next > nzumax ) {
 		    if (mem_error = zLUMemXpand(jcol, nextu, UCOL, &nzumax, Glu))
 			return (mem_error);
-		    ucol = Glu->ucol;
+		    ucol = (doublecomplex *) Glu->ucol;
 		    if (mem_error = zLUMemXpand(jcol, nextu, USUB, &nzumax, Glu))
 			return (mem_error);
 		    usub = Glu->usub;
