@@ -139,7 +139,6 @@ class LinearOperator(object):
                 raise TypeError("LinearOperator subclass should implement"
                                 " at least one of _matvec and _matmat.")
 
-            obj.__init__(*args, **kwargs)
             return obj
 
     def __init__(self, dtype, shape):
@@ -484,7 +483,9 @@ class _CustomLinearOperator(LinearOperator):
                                      dtype=self.dtype)
 
 
-def _get_dtype(operators, dtypes=[]):
+def _get_dtype(operators, dtypes=None):
+    if dtypes is None:
+        dtypes = []
     for obj in operators:
         if obj is not None and hasattr(obj, 'dtype'):
             dtypes.append(obj.dtype)
