@@ -1,6 +1,4 @@
 /**
- * distance_wrap.c
- *
  * Author: Damian Eads
  * Date:   September 22, 2007 (moved to new file on June 8, 2008)
  * Adapted for incorporation into Scipy, April 9, 2008.
@@ -34,111 +32,80 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+#if !defined(__clang__) && defined(__GNUC__) && defined(__GNUC_MINOR__)
+#if __GNUC__ >= 5 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 4)
+/* enable auto-vectorizer */
+#pragma GCC optimize("tree-vectorize")
+/* float associativity required to vectorize reductions */
+#pragma GCC optimize("unsafe-math-optimizations")
+/* maybe 5% gain, manual unrolling with more accumulators would be better */
+#pragma GCC optimize("unroll-loops")
+#endif
+#endif
+
 #include <math.h>
-#include "distance.h"
-#include "Python.h"
+#include <stdlib.h>
+#include <Python.h>
 #include <numpy/arrayobject.h>
-#include <stdio.h>
+#include <numpy/npy_math.h>
 
-extern PyObject *cdist_euclidean_wrap(PyObject *self, PyObject *args) {
-  PyArrayObject *XA_, *XB_, *dm_;
-  int mA, mB, n;
-  double *dm;
-  const double *XA, *XB;
-  if (!PyArg_ParseTuple(args, "O!O!O!",
-			&PyArray_Type, &XA_, &PyArray_Type, &XB_, 
-			&PyArray_Type, &dm_)) {
-    return 0;
-  }
-  else {
-    XA = (const double*)XA_->data;
-    XB = (const double*)XB_->data;
-    dm = (double*)dm_->data;
-    mA = XA_->dimensions[0];
-    mB = XB_->dimensions[0];
-    n = XA_->dimensions[1];
+#include "distance_impl.h"
 
-    cdist_euclidean(XA, XB, dm, mA, mB, n);
-  }
-  return Py_BuildValue("d", 0.0);
+#define DEFINE_WRAP_CDIST(name, type)                                   \
+    static PyObject *                                                   \
+    cdist_ ## name ## _wrap(PyObject *self, PyObject *args)             \
+    {                                                                   \
+        PyArrayObject *XA_, *XB_, *dm_;                                 \
+        Py_ssize_t mA, mB, n;                                           \
+        double *dm;                                                     \
+        const type *XA, *XB;                                            \
+        if (!PyArg_ParseTuple(args, "O!O!O!",                           \
+                              &PyArray_Type, &XA_, &PyArray_Type, &XB_, \
+                              &PyArray_Type, &dm_)) {                   \
+            return NULL;                                                \
+        }                                                               \
+        else {                                                          \
+            NPY_BEGIN_ALLOW_THREADS;                                    \
+            XA = (const type *)XA_->data;                               \
+            XB = (const type *)XB_->data;                               \
+            dm = (double *)dm_->data;                                   \
+            mA = XA_->dimensions[0];                                    \
+            mB = XB_->dimensions[0];                                    \
+            n = XA_->dimensions[1];                                     \
+            cdist_ ## name ## _ ## type(XA, XB, dm, mA, mB, n);         \
+            NPY_END_ALLOW_THREADS;                                      \
+        }                                                               \
+        return Py_BuildValue("d", 0.);                                  \
+    }
+
+DEFINE_WRAP_CDIST(bray_curtis, double)
+DEFINE_WRAP_CDIST(canberra, double)
+DEFINE_WRAP_CDIST(chebyshev, double)
+DEFINE_WRAP_CDIST(city_block, double)
+DEFINE_WRAP_CDIST(euclidean, double)
+DEFINE_WRAP_CDIST(hamming, double)
+DEFINE_WRAP_CDIST(jaccard, double)
+DEFINE_WRAP_CDIST(sqeuclidean, double)
+
+DEFINE_WRAP_CDIST(yule_bool, char)
+
+
+static NPY_INLINE double *mahalanobis_dimbuf(Py_ssize_t n)
+{
+    double *dimbuf;
+    dimbuf = calloc(n, 2 * sizeof(double));
+    if (!dimbuf) {
+        PyErr_Format(PyExc_MemoryError, "could not allocate %zd * %zd bytes",
+                     n, 2 * sizeof(double));
+    }
+    return dimbuf;
 }
 
 
-extern PyObject *cdist_sqeuclidean_wrap(PyObject *self, PyObject *args) {
-  PyArrayObject *XA_, *XB_, *dm_;
-  int mA, mB, n;
-  double *dm;
-  const double *XA, *XB;
-  if (!PyArg_ParseTuple(args, "O!O!O!",
-			&PyArray_Type, &XA_, &PyArray_Type, &XB_, 
-			&PyArray_Type, &dm_)) {
-    return 0;
-  }
-  else {
-    XA = (const double*)XA_->data;
-    XB = (const double*)XB_->data;
-    dm = (double*)dm_->data;
-    mA = XA_->dimensions[0];
-    mB = XB_->dimensions[0];
-    n = XA_->dimensions[1];
-
-    cdist_sqeuclidean(XA, XB, dm, mA, mB, n);
-  }
-  return Py_BuildValue("d", 0.0);
-}
-
-
-extern PyObject *cdist_canberra_wrap(PyObject *self, PyObject *args) {
-  PyArrayObject *XA_, *XB_, *dm_;
-  int mA, mB, n;
-  double *dm;
-  const double *XA, *XB;
-  if (!PyArg_ParseTuple(args, "O!O!O!",
-			&PyArray_Type, &XA_, &PyArray_Type, &XB_, 
-			&PyArray_Type, &dm_)) {
-    return 0;
-  }
-  else {
-    XA = (const double*)XA_->data;
-    XB = (const double*)XB_->data;
-    dm = (double*)dm_->data;
-    mA = XA_->dimensions[0];
-    mB = XB_->dimensions[0];
-    n = XA_->dimensions[1];
-
-    cdist_canberra(XA, XB, dm, mA, mB, n);
-  }
-  return Py_BuildValue("d", 0.0);
-}
-
-extern PyObject *cdist_bray_curtis_wrap(PyObject *self, PyObject *args) {
-  PyArrayObject *XA_, *XB_, *dm_;
-  int mA, mB, n;
-  double *dm;
-  const double *XA, *XB;
-  if (!PyArg_ParseTuple(args, "O!O!O!",
-			&PyArray_Type, &XA_, &PyArray_Type, &XB_, 
-			&PyArray_Type, &dm_)) {
-    return 0;
-  }
-  else {
-    XA = (const double*)XA_->data;
-    XB = (const double*)XB_->data;
-    dm = (double*)dm_->data;
-    mA = XA_->dimensions[0];
-    mB = XB_->dimensions[0];
-    n = XA_->dimensions[1];
-
-    cdist_bray_curtis(XA, XB, dm, mA, mB, n);
-  }
-  return Py_BuildValue("d", 0.0);
-}
-
-
-extern PyObject *cdist_mahalanobis_wrap(PyObject *self, PyObject *args) {
+static PyObject *cdist_mahalanobis_wrap(PyObject *self, PyObject *args) {
   PyArrayObject *XA_, *XB_, *covinv_, *dm_;
   int mA, mB, n;
-  double *dm;
+  double *dm, *dimbuf;
   const double *XA, *XB;
   const double *covinv;
   if (!PyArg_ParseTuple(args, "O!O!O!O!",
@@ -148,6 +115,8 @@ extern PyObject *cdist_mahalanobis_wrap(PyObject *self, PyObject *args) {
     return 0;
   }
   else {
+    NPY_BEGIN_THREADS_DEF;
+    NPY_BEGIN_THREADS;
     XA = (const double*)XA_->data;
     XB = (const double*)XB_->data;
     covinv = (const double*)covinv_->data;
@@ -156,64 +125,19 @@ extern PyObject *cdist_mahalanobis_wrap(PyObject *self, PyObject *args) {
     mB = XB_->dimensions[0];
     n = XA_->dimensions[1];
 
-    cdist_mahalanobis(XA, XB, covinv, dm, mA, mB, n);
+    dimbuf = mahalanobis_dimbuf(n);
+    if (!dimbuf) {
+      NPY_END_THREADS;
+      return NULL;
+    }
+    cdist_mahalanobis(XA, XB, covinv, dimbuf, dm, mA, mB, n);
+    free(dimbuf);
+    NPY_END_THREADS;
   }
   return Py_BuildValue("d", 0.0);
 }
 
-
-extern PyObject *cdist_chebyshev_wrap(PyObject *self, PyObject *args) {
-  PyArrayObject *XA_, *XB_, *dm_;
-  int mA, mB, n;
-  double *dm;
-  const double *XA, *XB;
-  if (!PyArg_ParseTuple(args, "O!O!O!",
-			&PyArray_Type, &XA_, &PyArray_Type, &XB_, 
-			&PyArray_Type, &dm_)) {
-    return 0;
-  }
-  else {
-    XA = (const double*)XA_->data;
-    XB = (const double*)XB_->data;
-    dm = (double*)dm_->data;
-    mA = XA_->dimensions[0];
-    mB = XB_->dimensions[0];
-    n = XA_->dimensions[1];
-
-    cdist_chebyshev(XA, XB, dm, mA, mB, n);
-  }
-  return Py_BuildValue("d", 0.0);
-}
-
-
-extern PyObject *cdist_cosine_wrap(PyObject *self, PyObject *args) {
-  PyArrayObject *XA_, *XB_, *dm_, *normsA_, *normsB_;
-  int mA, mB, n;
-  double *dm;
-  const double *XA, *XB, *normsA, *normsB;
-  if (!PyArg_ParseTuple(args, "O!O!O!O!O!",
-			&PyArray_Type, &XA_, &PyArray_Type, &XB_, 
-			&PyArray_Type, &dm_,
-			&PyArray_Type, &normsA_,
-			&PyArray_Type, &normsB_)) {
-    return 0;
-  }
-  else {
-    XA = (const double*)XA_->data;
-    XB = (const double*)XB_->data;
-    dm = (double*)dm_->data;
-    normsA = (const double*)normsA_->data;
-    normsB = (const double*)normsB_->data;
-    mA = XA_->dimensions[0];
-    mB = XB_->dimensions[0];
-    n = XA_->dimensions[1];
-
-    cdist_cosine(XA, XB, dm, mA, mB, n, normsA, normsB);
-  }
-  return Py_BuildValue("d", 0.0);
-}
-
-extern PyObject *cdist_seuclidean_wrap(PyObject *self, PyObject *args) {
+static PyObject *cdist_seuclidean_wrap(PyObject *self, PyObject *args) {
   PyArrayObject *XA_, *XB_, *dm_, *var_;
   int mA, mB, n;
   double *dm;
@@ -225,6 +149,7 @@ extern PyObject *cdist_seuclidean_wrap(PyObject *self, PyObject *args) {
     return 0;
   }
   else {
+    NPY_BEGIN_ALLOW_THREADS;
     XA = (const double*)XA_->data;
     XB = (const double*)XB_->data;
     dm = (double*)dm_->data;
@@ -234,57 +159,12 @@ extern PyObject *cdist_seuclidean_wrap(PyObject *self, PyObject *args) {
     n = XA_->dimensions[1];
 
     cdist_seuclidean(XA, XB, var, dm, mA, mB, n);
+    NPY_END_ALLOW_THREADS;
   }
   return Py_BuildValue("d", 0.0);
 }
 
-extern PyObject *cdist_city_block_wrap(PyObject *self, PyObject *args) {
-  PyArrayObject *XA_, *XB_, *dm_;
-  int mA, mB, n;
-  double *dm;
-  const double *XA, *XB;
-  if (!PyArg_ParseTuple(args, "O!O!O!",
-			&PyArray_Type, &XA_, &PyArray_Type, &XB_, 
-			&PyArray_Type, &dm_)) {
-    return 0;
-  }
-  else {
-    XA = (const double*)XA_->data;
-    XB = (const double*)XB_->data;
-    dm = (double*)dm_->data;
-    mA = XA_->dimensions[0];
-    mB = XB_->dimensions[0];
-    n = XA_->dimensions[1];
-
-    cdist_city_block(XA, XB, dm, mA, mB, n);
-  }
-  return Py_BuildValue("d", 0.0);
-}
-
-extern PyObject *cdist_hamming_wrap(PyObject *self, PyObject *args) {
-  PyArrayObject *XA_, *XB_, *dm_;
-  int mA, mB, n;
-  double *dm;
-  const double *XA, *XB;
-  if (!PyArg_ParseTuple(args, "O!O!O!",
-			&PyArray_Type, &XA_, &PyArray_Type, &XB_, 
-			&PyArray_Type, &dm_)) {
-    return 0;
-  }
-  else {
-    XA = (const double*)XA_->data;
-    XB = (const double*)XB_->data;
-    dm = (double*)dm_->data;
-    mA = XA_->dimensions[0];
-    mB = XB_->dimensions[0];
-    n = XA_->dimensions[1];
-
-    cdist_hamming(XA, XB, dm, mA, mB, n);
-  }
-  return Py_BuildValue("d", 0.0);
-}
-
-extern PyObject *cdist_hamming_bool_wrap(PyObject *self, PyObject *args) {
+static PyObject *cdist_hamming_bool_wrap(PyObject *self, PyObject *args) {
   PyArrayObject *XA_, *XB_, *dm_;
   int mA, mB, n;
   double *dm;
@@ -295,6 +175,7 @@ extern PyObject *cdist_hamming_bool_wrap(PyObject *self, PyObject *args) {
     return 0;
   }
   else {
+    NPY_BEGIN_ALLOW_THREADS;
     XA = (const char*)XA_->data;
     XB = (const char*)XB_->data;
     dm = (double*)dm_->data;
@@ -302,35 +183,13 @@ extern PyObject *cdist_hamming_bool_wrap(PyObject *self, PyObject *args) {
     mB = XB_->dimensions[0];
     n = XA_->dimensions[1];
 
-    cdist_hamming_bool(XA, XB, dm, mA, mB, n);
+    cdist_hamming_char(XA, XB, dm, mA, mB, n);
+    NPY_END_ALLOW_THREADS;
   }
   return Py_BuildValue("d", 0.0);
 }
 
-extern PyObject *cdist_jaccard_wrap(PyObject *self, PyObject *args) {
-  PyArrayObject *XA_, *XB_, *dm_;
-  int mA, mB, n;
-  double *dm;
-  const double *XA, *XB;
-  if (!PyArg_ParseTuple(args, "O!O!O!",
-			&PyArray_Type, &XA_, &PyArray_Type, &XB_, 
-			&PyArray_Type, &dm_)) {
-    return 0;
-  }
-  else {
-    XA = (const double*)XA_->data;
-    XB = (const double*)XB_->data;
-    dm = (double*)dm_->data;
-    mA = XA_->dimensions[0];
-    mB = XB_->dimensions[0];
-    n = XA_->dimensions[1];
-
-    cdist_jaccard(XA, XB, dm, mA, mB, n);
-  }
-  return Py_BuildValue("d", 0.0);
-}
-
-extern PyObject *cdist_jaccard_bool_wrap(PyObject *self, PyObject *args) {
+static PyObject *cdist_jaccard_bool_wrap(PyObject *self, PyObject *args) {
   PyArrayObject *XA_, *XB_, *dm_;
   int mA, mB, n;
   double *dm;
@@ -341,6 +200,7 @@ extern PyObject *cdist_jaccard_bool_wrap(PyObject *self, PyObject *args) {
     return 0;
   }
   else {
+    NPY_BEGIN_ALLOW_THREADS;
     XA = (const char*)XA_->data;
     XB = (const char*)XB_->data;
     dm = (double*)dm_->data;
@@ -348,12 +208,13 @@ extern PyObject *cdist_jaccard_bool_wrap(PyObject *self, PyObject *args) {
     mB = XB_->dimensions[0];
     n = XA_->dimensions[1];
 
-    cdist_jaccard_bool(XA, XB, dm, mA, mB, n);
+    cdist_jaccard_char(XA, XB, dm, mA, mB, n);
+    NPY_END_ALLOW_THREADS;
   }
   return Py_BuildValue("d", 0.0);
 }
 
-extern PyObject *cdist_minkowski_wrap(PyObject *self, PyObject *args) {
+static PyObject *cdist_minkowski_wrap(PyObject *self, PyObject *args) {
   PyArrayObject *XA_, *XB_, *dm_;
   int mA, mB, n;
   double *dm;
@@ -366,6 +227,7 @@ extern PyObject *cdist_minkowski_wrap(PyObject *self, PyObject *args) {
     return 0;
   }
   else {
+    NPY_BEGIN_ALLOW_THREADS;
     XA = (const double*)XA_->data;
     XB = (const double*)XB_->data;
     dm = (double*)dm_->data;
@@ -373,11 +235,12 @@ extern PyObject *cdist_minkowski_wrap(PyObject *self, PyObject *args) {
     mB = XB_->dimensions[0];
     n = XA_->dimensions[1];
     cdist_minkowski(XA, XB, dm, mA, mB, n, p);
+    NPY_END_ALLOW_THREADS;
   }
   return Py_BuildValue("d", 0.0);
 }
 
-extern PyObject *cdist_weighted_minkowski_wrap(PyObject *self, PyObject *args) {
+static PyObject *cdist_weighted_minkowski_wrap(PyObject *self, PyObject *args) {
   PyArrayObject *XA_, *XB_, *dm_, *w_;
   int mA, mB, n;
   double *dm;
@@ -391,6 +254,7 @@ extern PyObject *cdist_weighted_minkowski_wrap(PyObject *self, PyObject *args) {
     return 0;
   }
   else {
+    NPY_BEGIN_ALLOW_THREADS;
     XA = (const double*)XA_->data;
     XB = (const double*)XB_->data;
     w = (const double*)w_->data;
@@ -399,11 +263,12 @@ extern PyObject *cdist_weighted_minkowski_wrap(PyObject *self, PyObject *args) {
     mB = XB_->dimensions[0];
     n = XA_->dimensions[1];
     cdist_weighted_minkowski(XA, XB, dm, mA, mB, n, p, w);
+    NPY_END_ALLOW_THREADS;
   }
   return Py_BuildValue("d", 0.0);
 }
 
-extern PyObject *cdist_yule_bool_wrap(PyObject *self, PyObject *args) {
+static PyObject *cdist_dice_bool_wrap(PyObject *self, PyObject *args) {
   PyArrayObject *XA_, *XB_, *dm_;
   int mA, mB, n;
   double *dm;
@@ -414,6 +279,7 @@ extern PyObject *cdist_yule_bool_wrap(PyObject *self, PyObject *args) {
     return 0;
   }
   else {
+    NPY_BEGIN_ALLOW_THREADS;
     XA = (const char*)XA_->data;
     XB = (const char*)XB_->data;
     dm = (double*)dm_->data;
@@ -421,12 +287,13 @@ extern PyObject *cdist_yule_bool_wrap(PyObject *self, PyObject *args) {
     mB = XB_->dimensions[0];
     n = XA_->dimensions[1];
 
-    cdist_yule_bool(XA, XB, dm, mA, mB, n);
+    cdist_dice_char(XA, XB, dm, mA, mB, n);
+    NPY_END_ALLOW_THREADS;
   }
   return Py_BuildValue("");
 }
 
-extern PyObject *cdist_matching_bool_wrap(PyObject *self, PyObject *args) {
+static PyObject *cdist_rogerstanimoto_bool_wrap(PyObject *self, PyObject *args) {
   PyArrayObject *XA_, *XB_, *dm_;
   int mA, mB, n;
   double *dm;
@@ -437,6 +304,7 @@ extern PyObject *cdist_matching_bool_wrap(PyObject *self, PyObject *args) {
     return 0;
   }
   else {
+    NPY_BEGIN_ALLOW_THREADS;
     XA = (const char*)XA_->data;
     XB = (const char*)XB_->data;
     dm = (double*)dm_->data;
@@ -444,12 +312,13 @@ extern PyObject *cdist_matching_bool_wrap(PyObject *self, PyObject *args) {
     mB = XB_->dimensions[0];
     n = XA_->dimensions[1];
 
-    cdist_matching_bool(XA, XB, dm, mA, mB, n);
+    cdist_rogerstanimoto_char(XA, XB, dm, mA, mB, n);
+    NPY_END_ALLOW_THREADS;
   }
   return Py_BuildValue("");
 }
 
-extern PyObject *cdist_dice_bool_wrap(PyObject *self, PyObject *args) {
+static PyObject *cdist_russellrao_bool_wrap(PyObject *self, PyObject *args) {
   PyArrayObject *XA_, *XB_, *dm_;
   int mA, mB, n;
   double *dm;
@@ -460,6 +329,7 @@ extern PyObject *cdist_dice_bool_wrap(PyObject *self, PyObject *args) {
     return 0;
   }
   else {
+    NPY_BEGIN_ALLOW_THREADS;
     XA = (const char*)XA_->data;
     XB = (const char*)XB_->data;
     dm = (double*)dm_->data;
@@ -467,12 +337,13 @@ extern PyObject *cdist_dice_bool_wrap(PyObject *self, PyObject *args) {
     mB = XB_->dimensions[0];
     n = XA_->dimensions[1];
 
-    cdist_dice_bool(XA, XB, dm, mA, mB, n);
+    cdist_russellrao_char(XA, XB, dm, mA, mB, n);
+    NPY_END_ALLOW_THREADS;
   }
   return Py_BuildValue("");
 }
 
-extern PyObject *cdist_rogerstanimoto_bool_wrap(PyObject *self, PyObject *args) {
+static PyObject *cdist_kulsinski_bool_wrap(PyObject *self, PyObject *args) {
   PyArrayObject *XA_, *XB_, *dm_;
   int mA, mB, n;
   double *dm;
@@ -483,6 +354,7 @@ extern PyObject *cdist_rogerstanimoto_bool_wrap(PyObject *self, PyObject *args)
     return 0;
   }
   else {
+    NPY_BEGIN_ALLOW_THREADS;
     XA = (const char*)XA_->data;
     XB = (const char*)XB_->data;
     dm = (double*)dm_->data;
@@ -490,12 +362,13 @@ extern PyObject *cdist_rogerstanimoto_bool_wrap(PyObject *self, PyObject *args)
     mB = XB_->dimensions[0];
     n = XA_->dimensions[1];
 
-    cdist_rogerstanimoto_bool(XA, XB, dm, mA, mB, n);
+    cdist_kulsinski_char(XA, XB, dm, mA, mB, n);
+    NPY_END_ALLOW_THREADS;
   }
   return Py_BuildValue("");
 }
 
-extern PyObject *cdist_russellrao_bool_wrap(PyObject *self, PyObject *args) {
+static PyObject *cdist_sokalmichener_bool_wrap(PyObject *self, PyObject *args) {
   PyArrayObject *XA_, *XB_, *dm_;
   int mA, mB, n;
   double *dm;
@@ -506,6 +379,7 @@ extern PyObject *cdist_russellrao_bool_wrap(PyObject *self, PyObject *args) {
     return 0;
   }
   else {
+    NPY_BEGIN_ALLOW_THREADS;
     XA = (const char*)XA_->data;
     XB = (const char*)XB_->data;
     dm = (double*)dm_->data;
@@ -513,12 +387,13 @@ extern PyObject *cdist_russellrao_bool_wrap(PyObject *self, PyObject *args) {
     mB = XB_->dimensions[0];
     n = XA_->dimensions[1];
 
-    cdist_russellrao_bool(XA, XB, dm, mA, mB, n);
+    cdist_sokalmichener_char(XA, XB, dm, mA, mB, n);
+    NPY_END_ALLOW_THREADS;
   }
   return Py_BuildValue("");
 }
 
-extern PyObject *cdist_kulsinski_bool_wrap(PyObject *self, PyObject *args) {
+static PyObject *cdist_sokalsneath_bool_wrap(PyObject *self, PyObject *args) {
   PyArrayObject *XA_, *XB_, *dm_;
   int mA, mB, n;
   double *dm;
@@ -529,6 +404,7 @@ extern PyObject *cdist_kulsinski_bool_wrap(PyObject *self, PyObject *args) {
     return 0;
   }
   else {
+    NPY_BEGIN_ALLOW_THREADS;
     XA = (const char*)XA_->data;
     XB = (const char*)XB_->data;
     dm = (double*)dm_->data;
@@ -536,150 +412,52 @@ extern PyObject *cdist_kulsinski_bool_wrap(PyObject *self, PyObject *args) {
     mB = XB_->dimensions[0];
     n = XA_->dimensions[1];
 
-    cdist_kulsinski_bool(XA, XB, dm, mA, mB, n);
-  }
-  return Py_BuildValue("");
-}
-
-extern PyObject *cdist_sokalmichener_bool_wrap(PyObject *self, PyObject *args) {
-  PyArrayObject *XA_, *XB_, *dm_;
-  int mA, mB, n;
-  double *dm;
-  const char *XA, *XB;
-  if (!PyArg_ParseTuple(args, "O!O!O!",
-			&PyArray_Type, &XA_, &PyArray_Type, &XB_, 
-			&PyArray_Type, &dm_)) {
-    return 0;
-  }
-  else {
-    XA = (const char*)XA_->data;
-    XB = (const char*)XB_->data;
-    dm = (double*)dm_->data;
-    mA = XA_->dimensions[0];
-    mB = XB_->dimensions[0];
-    n = XA_->dimensions[1];
-
-    cdist_sokalmichener_bool(XA, XB, dm, mA, mB, n);
-  }
-  return Py_BuildValue("");
-}
-
-extern PyObject *cdist_sokalsneath_bool_wrap(PyObject *self, PyObject *args) {
-  PyArrayObject *XA_, *XB_, *dm_;
-  int mA, mB, n;
-  double *dm;
-  const char *XA, *XB;
-  if (!PyArg_ParseTuple(args, "O!O!O!",
-			&PyArray_Type, &XA_, &PyArray_Type, &XB_, 
-			&PyArray_Type, &dm_)) {
-    return 0;
-  }
-  else {
-    XA = (const char*)XA_->data;
-    XB = (const char*)XB_->data;
-    dm = (double*)dm_->data;
-    mA = XA_->dimensions[0];
-    mB = XB_->dimensions[0];
-    n = XA_->dimensions[1];
-
-    cdist_sokalsneath_bool(XA, XB, dm, mA, mB, n);
+    cdist_sokalsneath_char(XA, XB, dm, mA, mB, n);
+    NPY_END_ALLOW_THREADS;
   }
   return Py_BuildValue("");
 }
 
 /***************************** pdist ***/
 
-extern PyObject *pdist_euclidean_wrap(PyObject *self, PyObject *args) {
-  PyArrayObject *X_, *dm_;
-  int m, n;
-  double *dm;
-  const double *X;
-  if (!PyArg_ParseTuple(args, "O!O!",
-			&PyArray_Type, &X_,
-			&PyArray_Type, &dm_)) {
-    return 0;
-  }
-  else {
-    X = (const double*)X_->data;
-    dm = (double*)dm_->data;
-    m = X_->dimensions[0];
-    n = X_->dimensions[1];
+#define DEFINE_WRAP_PDIST_DOUBLE(name)                                  \
+    static PyObject *                                                   \
+    pdist_ ## name ## _wrap(PyObject *self, PyObject *args)             \
+    {                                                                   \
+        PyArrayObject *X_, *dm_;                                        \
+        Py_ssize_t m, n;                                                \
+        double *dm;                                                     \
+        const double *X;                                                \
+        if (!PyArg_ParseTuple(args, "O!O!", &PyArray_Type, &X_,         \
+                                            &PyArray_Type, &dm_)) {     \
+            return NULL;                                                \
+        }                                                               \
+        else {                                                          \
+            NPY_BEGIN_ALLOW_THREADS;                                    \
+            X = (const double *)X_->data;                               \
+            dm = (double *)dm_->data;                                   \
+            m = X_->dimensions[0];                                      \
+            n = X_->dimensions[1];                                      \
+            pdist_ ## name ## _double(X, dm, m, n);                     \
+            NPY_END_ALLOW_THREADS;                                      \
+        }                                                               \
+        return Py_BuildValue("d", 0.);                                  \
+    }
 
-    pdist_euclidean(X, dm, m, n);
-  }
-  return Py_BuildValue("d", 0.0);
-}
+DEFINE_WRAP_PDIST_DOUBLE(bray_curtis)
+DEFINE_WRAP_PDIST_DOUBLE(canberra)
+DEFINE_WRAP_PDIST_DOUBLE(chebyshev)
+DEFINE_WRAP_PDIST_DOUBLE(city_block)
+DEFINE_WRAP_PDIST_DOUBLE(euclidean)
+DEFINE_WRAP_PDIST_DOUBLE(hamming)
+DEFINE_WRAP_PDIST_DOUBLE(jaccard)
+DEFINE_WRAP_PDIST_DOUBLE(sqeuclidean)
 
 
-extern PyObject *pdist_sqeuclidean_wrap(PyObject *self, PyObject *args) {
-  PyArrayObject *X_, *dm_;
-  int m, n;
-  double *dm;
-  const double *X;
-  if (!PyArg_ParseTuple(args, "O!O!",
-			&PyArray_Type, &X_,
-			&PyArray_Type, &dm_)) {
-    return 0;
-  }
-  else {
-    X = (const double*)X_->data;
-    dm = (double*)dm_->data;
-    m = X_->dimensions[0];
-    n = X_->dimensions[1];
-
-    pdist_sqeuclidean(X, dm, m, n);
-  }
-  return Py_BuildValue("d", 0.0);
-}
-
-
-extern PyObject *pdist_canberra_wrap(PyObject *self, PyObject *args) {
-  PyArrayObject *X_, *dm_;
-  int m, n;
-  double *dm;
-  const double *X;
-  if (!PyArg_ParseTuple(args, "O!O!",
-			&PyArray_Type, &X_,
-			&PyArray_Type, &dm_)) {
-    return 0;
-  }
-  else {
-    X = (const double*)X_->data;
-    dm = (double*)dm_->data;
-    m = X_->dimensions[0];
-    n = X_->dimensions[1];
-
-    pdist_canberra(X, dm, m, n);
-  }
-  return Py_BuildValue("d", 0.0);
-}
-
-extern PyObject *pdist_bray_curtis_wrap(PyObject *self, PyObject *args) {
-  PyArrayObject *X_, *dm_;
-  int m, n;
-  double *dm;
-  const double *X;
-  if (!PyArg_ParseTuple(args, "O!O!",
-			&PyArray_Type, &X_,
-			&PyArray_Type, &dm_)) {
-    return 0;
-  }
-  else {
-    X = (const double*)X_->data;
-    dm = (double*)dm_->data;
-    m = X_->dimensions[0];
-    n = X_->dimensions[1];
-
-    pdist_bray_curtis(X, dm, m, n);
-  }
-  return Py_BuildValue("d", 0.0);
-}
-
-
-extern PyObject *pdist_mahalanobis_wrap(PyObject *self, PyObject *args) {
+static PyObject *pdist_mahalanobis_wrap(PyObject *self, PyObject *args) {
   PyArrayObject *X_, *covinv_, *dm_;
   int m, n;
-  double *dm;
+  double *dimbuf, *dm;
   const double *X;
   const double *covinv;
   if (!PyArg_ParseTuple(args, "O!O!O!",
@@ -689,41 +467,29 @@ extern PyObject *pdist_mahalanobis_wrap(PyObject *self, PyObject *args) {
     return 0;
   }
   else {
+    NPY_BEGIN_THREADS_DEF;
+    NPY_BEGIN_THREADS;
     X = (const double*)X_->data;
     covinv = (const double*)covinv_->data;
     dm = (double*)dm_->data;
     m = X_->dimensions[0];
     n = X_->dimensions[1];
 
-    pdist_mahalanobis(X, covinv, dm, m, n);
+    dimbuf = mahalanobis_dimbuf(n);
+    if (!dimbuf) {
+        NPY_END_THREADS;
+        return NULL;
+    }
+
+    pdist_mahalanobis(X, covinv, dimbuf, dm, m, n);
+    free(dimbuf);
+    NPY_END_THREADS;
   }
   return Py_BuildValue("d", 0.0);
 }
 
 
-extern PyObject *pdist_chebyshev_wrap(PyObject *self, PyObject *args) {
-  PyArrayObject *X_, *dm_;
-  int m, n;
-  double *dm;
-  const double *X;
-  if (!PyArg_ParseTuple(args, "O!O!",
-			&PyArray_Type, &X_,
-			&PyArray_Type, &dm_)) {
-    return 0;
-  }
-  else {
-    X = (const double*)X_->data;
-    dm = (double*)dm_->data;
-    m = X_->dimensions[0];
-    n = X_->dimensions[1];
-
-    pdist_chebyshev(X, dm, m, n);
-  }
-  return Py_BuildValue("d", 0.0);
-}
-
-
-extern PyObject *pdist_cosine_wrap(PyObject *self, PyObject *args) {
+static PyObject *pdist_cosine_wrap(PyObject *self, PyObject *args) {
   PyArrayObject *X_, *dm_, *norms_;
   int m, n;
   double *dm;
@@ -735,6 +501,7 @@ extern PyObject *pdist_cosine_wrap(PyObject *self, PyObject *args) {
     return 0;
   }
   else {
+    NPY_BEGIN_ALLOW_THREADS;
     X = (const double*)X_->data;
     dm = (double*)dm_->data;
     norms = (const double*)norms_->data;
@@ -742,11 +509,12 @@ extern PyObject *pdist_cosine_wrap(PyObject *self, PyObject *args) {
     n = X_->dimensions[1];
 
     pdist_cosine(X, dm, m, n, norms);
+    NPY_END_ALLOW_THREADS;
   }
   return Py_BuildValue("d", 0.0);
 }
 
-extern PyObject *pdist_seuclidean_wrap(PyObject *self, PyObject *args) {
+static PyObject *pdist_seuclidean_wrap(PyObject *self, PyObject *args) {
   PyArrayObject *X_, *dm_, *var_;
   int m, n;
   double *dm;
@@ -758,6 +526,7 @@ extern PyObject *pdist_seuclidean_wrap(PyObject *self, PyObject *args) {
     return 0;
   }
   else {
+    NPY_BEGIN_ALLOW_THREADS;
     X = (double*)X_->data;
     dm = (double*)dm_->data;
     var = (double*)var_->data;
@@ -765,53 +534,12 @@ extern PyObject *pdist_seuclidean_wrap(PyObject *self, PyObject *args) {
     n = X_->dimensions[1];
 
     pdist_seuclidean(X, var, dm, m, n);
+    NPY_END_ALLOW_THREADS;
   }
   return Py_BuildValue("d", 0.0);
 }
 
-extern PyObject *pdist_city_block_wrap(PyObject *self, PyObject *args) {
-  PyArrayObject *X_, *dm_;
-  int m, n;
-  double *dm;
-  const double *X;
-  if (!PyArg_ParseTuple(args, "O!O!",
-			&PyArray_Type, &X_,
-			&PyArray_Type, &dm_)) {
-    return 0;
-  }
-  else {
-    X = (const double*)X_->data;
-    dm = (double*)dm_->data;
-    m = X_->dimensions[0];
-    n = X_->dimensions[1];
-
-    pdist_city_block(X, dm, m, n);
-  }
-  return Py_BuildValue("d", 0.0);
-}
-
-extern PyObject *pdist_hamming_wrap(PyObject *self, PyObject *args) {
-  PyArrayObject *X_, *dm_;
-  int m, n;
-  double *dm;
-  const double *X;
-  if (!PyArg_ParseTuple(args, "O!O!",
-			&PyArray_Type, &X_,
-			&PyArray_Type, &dm_)) {
-    return 0;
-  }
-  else {
-    X = (const double*)X_->data;
-    dm = (double*)dm_->data;
-    m = X_->dimensions[0];
-    n = X_->dimensions[1];
-
-    pdist_hamming(X, dm, m, n);
-  }
-  return Py_BuildValue("d", 0.0);
-}
-
-extern PyObject *pdist_hamming_bool_wrap(PyObject *self, PyObject *args) {
+static PyObject *pdist_hamming_bool_wrap(PyObject *self, PyObject *args) {
   PyArrayObject *X_, *dm_;
   int m, n;
   double *dm;
@@ -822,38 +550,19 @@ extern PyObject *pdist_hamming_bool_wrap(PyObject *self, PyObject *args) {
     return 0;
   }
   else {
+    NPY_BEGIN_ALLOW_THREADS;
     X = (const char*)X_->data;
     dm = (double*)dm_->data;
     m = X_->dimensions[0];
     n = X_->dimensions[1];
 
-    pdist_hamming_bool(X, dm, m, n);
+    pdist_hamming_char(X, dm, m, n);
+    NPY_END_ALLOW_THREADS;
   }
   return Py_BuildValue("d", 0.0);
 }
 
-extern PyObject *pdist_jaccard_wrap(PyObject *self, PyObject *args) {
-  PyArrayObject *X_, *dm_;
-  int m, n;
-  double *dm;
-  const double *X;
-  if (!PyArg_ParseTuple(args, "O!O!",
-			&PyArray_Type, &X_,
-			&PyArray_Type, &dm_)) {
-    return 0;
-  }
-  else {
-    X = (const double*)X_->data;
-    dm = (double*)dm_->data;
-    m = X_->dimensions[0];
-    n = X_->dimensions[1];
-
-    pdist_jaccard(X, dm, m, n);
-  }
-  return Py_BuildValue("d", 0.0);
-}
-
-extern PyObject *pdist_jaccard_bool_wrap(PyObject *self, PyObject *args) {
+static PyObject *pdist_jaccard_bool_wrap(PyObject *self, PyObject *args) {
   PyArrayObject *X_, *dm_;
   int m, n;
   double *dm;
@@ -864,17 +573,19 @@ extern PyObject *pdist_jaccard_bool_wrap(PyObject *self, PyObject *args) {
     return 0;
   }
   else {
+    NPY_BEGIN_ALLOW_THREADS;
     X = (const char*)X_->data;
     dm = (double*)dm_->data;
     m = X_->dimensions[0];
     n = X_->dimensions[1];
 
-    pdist_jaccard_bool(X, dm, m, n);
+    pdist_jaccard_char(X, dm, m, n);
+    NPY_END_ALLOW_THREADS;
   }
   return Py_BuildValue("d", 0.0);
 }
 
-extern PyObject *pdist_minkowski_wrap(PyObject *self, PyObject *args) {
+static PyObject *pdist_minkowski_wrap(PyObject *self, PyObject *args) {
   PyArrayObject *X_, *dm_;
   int m, n;
   double *dm, *X;
@@ -886,17 +597,19 @@ extern PyObject *pdist_minkowski_wrap(PyObject *self, PyObject *args) {
     return 0;
   }
   else {
+    NPY_BEGIN_ALLOW_THREADS;
     X = (double*)X_->data;
     dm = (double*)dm_->data;
     m = X_->dimensions[0];
     n = X_->dimensions[1];
 
     pdist_minkowski(X, dm, m, n, p);
+    NPY_END_ALLOW_THREADS;
   }
   return Py_BuildValue("d", 0.0);
 }
 
-extern PyObject *pdist_weighted_minkowski_wrap(PyObject *self, PyObject *args) {
+static PyObject *pdist_weighted_minkowski_wrap(PyObject *self, PyObject *args) {
   PyArrayObject *X_, *dm_, *w_;
   int m, n;
   double *dm, *X, *w;
@@ -909,19 +622,21 @@ extern PyObject *pdist_weighted_minkowski_wrap(PyObject *self, PyObject *args) {
     return 0;
   }
   else {
+    NPY_BEGIN_ALLOW_THREADS;
     X = (double*)X_->data;
     dm = (double*)dm_->data;
-    w = (const double*)w_->data;
+    w = (double*)w_->data;
     m = X_->dimensions[0];
     n = X_->dimensions[1];
 
     pdist_weighted_minkowski(X, dm, m, n, p, w);
+    NPY_END_ALLOW_THREADS;
   }
   return Py_BuildValue("d", 0.0);
 }
 
 
-extern PyObject *pdist_yule_bool_wrap(PyObject *self, PyObject *args) {
+static PyObject *pdist_yule_bool_wrap(PyObject *self, PyObject *args) {
   PyArrayObject *X_, *dm_;
   int m, n;
   double *dm;
@@ -932,17 +647,19 @@ extern PyObject *pdist_yule_bool_wrap(PyObject *self, PyObject *args) {
     return 0;
   }
   else {
+    NPY_BEGIN_ALLOW_THREADS;
     X = (const char*)X_->data;
     dm = (double*)dm_->data;
     m = X_->dimensions[0];
     n = X_->dimensions[1];
 
-    pdist_yule_bool(X, dm, m, n);
+    pdist_yule_bool_char(X, dm, m, n);
+    NPY_END_ALLOW_THREADS;
   }
   return Py_BuildValue("");
 }
 
-extern PyObject *pdist_matching_bool_wrap(PyObject *self, PyObject *args) {
+static PyObject *pdist_dice_bool_wrap(PyObject *self, PyObject *args) {
   PyArrayObject *X_, *dm_;
   int m, n;
   double *dm;
@@ -953,17 +670,19 @@ extern PyObject *pdist_matching_bool_wrap(PyObject *self, PyObject *args) {
     return 0;
   }
   else {
+    NPY_BEGIN_ALLOW_THREADS;
     X = (const char*)X_->data;
     dm = (double*)dm_->data;
     m = X_->dimensions[0];
     n = X_->dimensions[1];
 
-    pdist_matching_bool(X, dm, m, n);
+    pdist_dice_char(X, dm, m, n);
+    NPY_END_ALLOW_THREADS;
   }
   return Py_BuildValue("");
 }
 
-extern PyObject *pdist_dice_bool_wrap(PyObject *self, PyObject *args) {
+static PyObject *pdist_rogerstanimoto_bool_wrap(PyObject *self, PyObject *args) {
   PyArrayObject *X_, *dm_;
   int m, n;
   double *dm;
@@ -974,17 +693,19 @@ extern PyObject *pdist_dice_bool_wrap(PyObject *self, PyObject *args) {
     return 0;
   }
   else {
+    NPY_BEGIN_ALLOW_THREADS;
     X = (const char*)X_->data;
     dm = (double*)dm_->data;
     m = X_->dimensions[0];
     n = X_->dimensions[1];
 
-    pdist_dice_bool(X, dm, m, n);
+    pdist_rogerstanimoto_char(X, dm, m, n);
+    NPY_END_ALLOW_THREADS;
   }
   return Py_BuildValue("");
 }
 
-extern PyObject *pdist_rogerstanimoto_bool_wrap(PyObject *self, PyObject *args) {
+static PyObject *pdist_russellrao_bool_wrap(PyObject *self, PyObject *args) {
   PyArrayObject *X_, *dm_;
   int m, n;
   double *dm;
@@ -995,17 +716,19 @@ extern PyObject *pdist_rogerstanimoto_bool_wrap(PyObject *self, PyObject *args)
     return 0;
   }
   else {
+    NPY_BEGIN_ALLOW_THREADS;
     X = (const char*)X_->data;
     dm = (double*)dm_->data;
     m = X_->dimensions[0];
     n = X_->dimensions[1];
 
-    pdist_rogerstanimoto_bool(X, dm, m, n);
+    pdist_russellrao_char(X, dm, m, n);
+    NPY_END_ALLOW_THREADS;
   }
   return Py_BuildValue("");
 }
 
-extern PyObject *pdist_russellrao_bool_wrap(PyObject *self, PyObject *args) {
+static PyObject *pdist_kulsinski_bool_wrap(PyObject *self, PyObject *args) {
   PyArrayObject *X_, *dm_;
   int m, n;
   double *dm;
@@ -1016,17 +739,19 @@ extern PyObject *pdist_russellrao_bool_wrap(PyObject *self, PyObject *args) {
     return 0;
   }
   else {
+    NPY_BEGIN_ALLOW_THREADS;
     X = (const char*)X_->data;
     dm = (double*)dm_->data;
     m = X_->dimensions[0];
     n = X_->dimensions[1];
 
-    pdist_russellrao_bool(X, dm, m, n);
+    pdist_kulsinski_char(X, dm, m, n);
+    NPY_END_ALLOW_THREADS;
   }
   return Py_BuildValue("");
 }
 
-extern PyObject *pdist_kulsinski_bool_wrap(PyObject *self, PyObject *args) {
+static PyObject *pdist_sokalmichener_bool_wrap(PyObject *self, PyObject *args) {
   PyArrayObject *X_, *dm_;
   int m, n;
   double *dm;
@@ -1037,17 +762,19 @@ extern PyObject *pdist_kulsinski_bool_wrap(PyObject *self, PyObject *args) {
     return 0;
   }
   else {
+    NPY_BEGIN_ALLOW_THREADS;
     X = (const char*)X_->data;
     dm = (double*)dm_->data;
     m = X_->dimensions[0];
     n = X_->dimensions[1];
 
-    pdist_kulsinski_bool(X, dm, m, n);
+    pdist_sokalmichener_char(X, dm, m, n);
+    NPY_END_ALLOW_THREADS;
   }
   return Py_BuildValue("");
 }
 
-extern PyObject *pdist_sokalmichener_bool_wrap(PyObject *self, PyObject *args) {
+static PyObject *pdist_sokalsneath_bool_wrap(PyObject *self, PyObject *args) {
   PyArrayObject *X_, *dm_;
   int m, n;
   double *dm;
@@ -1058,38 +785,19 @@ extern PyObject *pdist_sokalmichener_bool_wrap(PyObject *self, PyObject *args) {
     return 0;
   }
   else {
+    NPY_BEGIN_ALLOW_THREADS;
     X = (const char*)X_->data;
     dm = (double*)dm_->data;
     m = X_->dimensions[0];
     n = X_->dimensions[1];
 
-    pdist_sokalmichener_bool(X, dm, m, n);
+    pdist_sokalsneath_char(X, dm, m, n);
+    NPY_END_ALLOW_THREADS;
   }
   return Py_BuildValue("");
 }
 
-extern PyObject *pdist_sokalsneath_bool_wrap(PyObject *self, PyObject *args) {
-  PyArrayObject *X_, *dm_;
-  int m, n;
-  double *dm;
-  const char *X;
-  if (!PyArg_ParseTuple(args, "O!O!",
-			&PyArray_Type, &X_,
-			&PyArray_Type, &dm_)) {
-    return 0;
-  }
-  else {
-    X = (const char*)X_->data;
-    dm = (double*)dm_->data;
-    m = X_->dimensions[0];
-    n = X_->dimensions[1];
-
-    pdist_sokalsneath_bool(X, dm, m, n);
-  }
-  return Py_BuildValue("");
-}
-
-extern PyObject *to_squareform_from_vector_wrap(PyObject *self, PyObject *args) {
+static PyObject *to_squareform_from_vector_wrap(PyObject *self, PyObject *args) {
   PyArrayObject *M_, *v_;
   int n;
   const double *v;
@@ -1100,15 +808,17 @@ extern PyObject *to_squareform_from_vector_wrap(PyObject *self, PyObject *args)
     return 0;
   }
   else {
+    NPY_BEGIN_ALLOW_THREADS;
     M = (double*)M_->data;
     v = (const double*)v_->data;
     n = M_->dimensions[0];
     dist_to_squareform_from_vector(M, v, n);
+    NPY_END_ALLOW_THREADS;
   }
   return Py_BuildValue("d", 0.0);
 }
 
-extern PyObject *to_vector_from_squareform_wrap(PyObject *self, PyObject *args) {
+static PyObject *to_vector_from_squareform_wrap(PyObject *self, PyObject *args) {
   PyArrayObject *M_, *v_;
   int n;
   double *v;
@@ -1119,10 +829,12 @@ extern PyObject *to_vector_from_squareform_wrap(PyObject *self, PyObject *args)
     return 0;
   }
   else {
+    NPY_BEGIN_ALLOW_THREADS;
     M = (const double*)M_->data;
     v = (double*)v_->data;
     n = M_->dimensions[0];
     dist_to_vector_from_squareform(M, v, n);
+    NPY_END_ALLOW_THREADS;
   }
   return Py_BuildValue("d", 0.0);
 }
@@ -1133,7 +845,6 @@ static PyMethodDef _distanceWrapMethods[] = {
   {"cdist_canberra_wrap", cdist_canberra_wrap, METH_VARARGS},
   {"cdist_chebyshev_wrap", cdist_chebyshev_wrap, METH_VARARGS},
   {"cdist_city_block_wrap", cdist_city_block_wrap, METH_VARARGS},
-  {"cdist_cosine_wrap", cdist_cosine_wrap, METH_VARARGS},
   {"cdist_dice_bool_wrap", cdist_dice_bool_wrap, METH_VARARGS},
   {"cdist_euclidean_wrap", cdist_euclidean_wrap, METH_VARARGS},
   {"cdist_sqeuclidean_wrap", cdist_sqeuclidean_wrap, METH_VARARGS},
@@ -1143,7 +854,6 @@ static PyMethodDef _distanceWrapMethods[] = {
   {"cdist_jaccard_bool_wrap", cdist_jaccard_bool_wrap, METH_VARARGS},
   {"cdist_kulsinski_bool_wrap", cdist_kulsinski_bool_wrap, METH_VARARGS},
   {"cdist_mahalanobis_wrap", cdist_mahalanobis_wrap, METH_VARARGS},
-  {"cdist_matching_bool_wrap", cdist_matching_bool_wrap, METH_VARARGS},
   {"cdist_minkowski_wrap", cdist_minkowski_wrap, METH_VARARGS},
   {"cdist_weighted_minkowski_wrap", cdist_weighted_minkowski_wrap, METH_VARARGS},
   {"cdist_rogerstanimoto_bool_wrap", cdist_rogerstanimoto_bool_wrap, METH_VARARGS},
@@ -1166,7 +876,6 @@ static PyMethodDef _distanceWrapMethods[] = {
   {"pdist_jaccard_bool_wrap", pdist_jaccard_bool_wrap, METH_VARARGS},
   {"pdist_kulsinski_bool_wrap", pdist_kulsinski_bool_wrap, METH_VARARGS},
   {"pdist_mahalanobis_wrap", pdist_mahalanobis_wrap, METH_VARARGS},
-  {"pdist_matching_bool_wrap", pdist_matching_bool_wrap, METH_VARARGS},
   {"pdist_minkowski_wrap", pdist_minkowski_wrap, METH_VARARGS},
   {"pdist_weighted_minkowski_wrap", pdist_weighted_minkowski_wrap, METH_VARARGS},
   {"pdist_rogerstanimoto_bool_wrap", pdist_rogerstanimoto_bool_wrap, METH_VARARGS},
