@@ -101,7 +101,6 @@ class test_random(ConsistencyTests):
         self.d = 0.2
         self.k = 10
 
-
 class test_random_far(test_random):
     def setUp(self):
         test_random.setUp(self)
@@ -1004,6 +1003,40 @@ def test_ckdtree_view():
 # cKDTree is specialized to type double points, so no need to make
 # a unit test corresponding to test_ball_point_ints()
 
+def test_ckdtree_list_k():
+    # check ckdtree periodic boundary
+    n = 200
+    m = 2
+    klist = [1, 2, 3]
+    kint = 3
+
+    np.random.seed(1234)
+    data = np.random.uniform(size=(n, m))
+    kdtree = cKDTree(data, leafsize=1)
+
+    # check agreement between arange(1,k+1) and k
+    dd, ii = kdtree.query(data, klist)
+    dd1, ii1 = kdtree.query(data, kint)
+    assert_equal(dd, dd1)
+    assert_equal(ii, ii1)
+
+    # now check skipping one element
+    klist = np.array([1, 3])
+    kint = 3
+    dd, ii = kdtree.query(data, kint)
+    dd1, ii1 = kdtree.query(data, klist)
+    assert_equal(dd1, dd[..., klist - 1])
+    assert_equal(ii1, ii[..., klist - 1])
+
+    # check k == 1 special case
+    # and k == [1] non-special case
+    dd, ii = kdtree.query(data, 1)
+    dd1, ii1 = kdtree.query(data, [1])
+    assert_equal(len(dd.shape), 1)
+    assert_equal(len(dd1.shape), 2)
+    assert_equal(dd, np.ravel(dd1))
+    assert_equal(ii, np.ravel(ii1))
+
 def test_ckdtree_box():
     # check ckdtree periodic boundary
     n = 2000
@@ -1086,7 +1119,7 @@ def test_ckdtree_memuse():
     z_copy[:] = z
     # Place FILLVAL in z_copy at random number of random locations
     FILLVAL = 99.
-    mask = np.random.random_integers(0, z.size - 1, np.random.randint(50) + 5)
+    mask = np.random.randint(0, z.size, np.random.randint(50) + 5)
     z_copy.flat[mask] = FILLVAL
     igood = np.vstack(np.where(x != FILLVAL)).T
     ibad = np.vstack(np.where(x == FILLVAL)).T
@@ -1107,5 +1140,76 @@ def test_ckdtree_memuse():
     # outside cKDTree
     assert_(num_leaks < 10)
     
+def test_len0_arrays():
+    # make sure len-0 arrays are handled correctly
+    # in range queries (gh-5639)
+    np.random.seed(1234)
+    X = np.random.rand(10,2)
+    Y = np.random.rand(10,2)
+    tree = cKDTree(X)
+    # query_ball_point (single)
+    d,i = tree.query([.5, .5], k=1)
+    z = tree.query_ball_point([.5, .5], 0.1*d)
+    assert_array_equal(z, [])
+    # query_ball_point (multiple)
+    d,i = tree.query(Y, k=1)
+    mind = d.min()
+    z = tree.query_ball_point(Y, 0.1*mind)
+    y = np.empty(shape=(10,), dtype=object)
+    y.fill([])
+    assert_array_equal(y, z)
+    # query_ball_tree
+    other = cKDTree(Y)
+    y = tree.query_ball_tree(other, 0.1*mind)
+    assert_array_equal(10*[[]], y)
+    # count_neighbors
+    y = tree.count_neighbors(other, 0.1*mind)
+    assert_(y == 0)
+    # sparse_distance_matrix
+    y = tree.sparse_distance_matrix(other, 0.1*mind, output_type='dok_matrix')
+    assert_array_equal(y == np.zeros((10,10)), True)
+    y = tree.sparse_distance_matrix(other, 0.1*mind, output_type='coo_matrix')
+    assert_array_equal(y == np.zeros((10,10)), True)
+    y = tree.sparse_distance_matrix(other, 0.1*mind, output_type='dict')
+    assert_equal(y, {})
+    y = tree.sparse_distance_matrix(other,0.1*mind, output_type='ndarray')
+    _dtype = [('i',np.intp), ('j',np.intp), ('v',np.float64)]
+    res_dtype = np.dtype(_dtype, align=True)
+    z = np.empty(shape=(0,), dtype=res_dtype)
+    assert_array_equal(y, z)
+    # query_pairs
+    d,i = tree.query(X, k=2)
+    mind = d[:,-1].min()
+    y = tree.query_pairs(0.1*mind, output_type='set')
+    assert_equal(y, set())
+    y = tree.query_pairs(0.1*mind, output_type='ndarray')
+    z = np.empty(shape=(0,2), dtype=np.intp)
+    assert_array_equal(y, z)
+
+def test_short_knn():
+
+    # The test case is based on github: #6425 by @SteveDoyle2
+
+    xyz = np.array([
+        [0., 0., 0.],
+        [1.01, 0., 0.],
+        [0., 1., 0.],
+        [0., 1.01, 0.],
+        [1., 0., 0.],
+        [1., 1., 0.],],
+    dtype='float64')
+
+    ckdt = cKDTree(xyz)
+
+    deq, ieq = ckdt.query(xyz, k=4, distance_upper_bound=0.2)
+
+    assert_array_almost_equal(deq,
+            [[0., np.inf, np.inf, np.inf],
+            [0., 0.01, np.inf, np.inf],
+            [0., 0.01, np.inf, np.inf],
+            [0., 0.01, np.inf, np.inf],
+            [0., 0.01, np.inf, np.inf],
+            [0., np.inf, np.inf, np.inf]])
+
 if __name__ == "__main__":
     run_module_suite()
