@@ -20,10 +20,10 @@ typedef enum {
 } sf_error_t;
 
 extern const char *sf_error_messages[];
-void sf_error(char *func_name, sf_error_t code, char *fmt, ...);
-void sf_error_check_fpe(char *func_name);
+void sf_error(const char *func_name, sf_error_t code, const char *fmt, ...);
+void sf_error_check_fpe(const char *func_name);
 int sf_error_set_print(int flag);
-int sf_error_get_print();
+int sf_error_get_print(void);
 
 #ifdef __cplusplus
 }
