@@ -34,6 +34,7 @@ NAME_MAP = {
     u('alex'): u('Alex Griffing'),
     u('aman-thakral'): u('Aman Thakral'),
     u('andbo'): u('Anders Bech Borchersen'),
+    u('andrew'): u('Andrew Fowlie'),
     u('Andreea_G'): u('Andreea Georgescu'),
     u('Andreas H'): u('Andreas Hilboll'),
     u('argriffing'): u('Alex Griffing'),
@@ -41,10 +42,13 @@ NAME_MAP = {
     u('ArmstrongJ'): u('Jeff Armstrong'),
     u('aw'): u('@chemelnucfin'),
     u('axiru'): u('@axiru'),
+    u('bdvd'): u('Dávid Bodnár'),
+    u('bhavikat'): u('Bhavika Tekwani'),
     u('Benny'): u('Benny Malengier'),
     u('behzad nouri'): u('Behzad Nouri'),
     u('bewithaman'): u('Aman Singh'),
     u('brettrmurphy'): u('Brett R. Murphy'),
+    u('Charles Nathan Woods'): u('Nathan Woods'),
     u('cel4'): u('@cel4'),
     u('cgholke'): u('Christoph Gohlke'),
     u('cgohlke'): u('Christoph Gohlke'),
@@ -68,14 +72,19 @@ NAME_MAP = {
     u('e-q'): u('Eric Quintero'),
     u('Eric89GXL'): u('Eric Larson'),
     u('Gael varoquaux'): u('Gaël Varoquaux'),
+    u('Garrett-R'): u('Garrett Reynolds'),
+    u('gfyoung'): u('G Young'),
     u('giorgiop'): u('Giorgio Patrini'),
     u('gotgenes'): u('Chris Lasher'),
+    u('golnazir'): u('Golnaz Irannejad'),
     u('Han'): u('Han Genuit'),
     u('Helder'): u('Helder Cesar'),
     u('HelmutAIT'): u('Helmut Toplitzer'),
+    u('hlin117'): u('Henry Lin'),
     u('honnorat'): u('Marc Honnorat'),
     u('Horta'): u('Danilo Horta'),
     u('I--P'): u('Irvin Probst'),
+    u('Ion'): u('Ion Elberdin'),
     u('Jake Vanderplas'): u('Jacob Vanderplas'),
     u('Jake VanderPlas'): u('Jacob Vanderplas'),
     u('jamestwebber'): u('James T. Webber'),
@@ -83,6 +92,9 @@ NAME_MAP = {
     u('Jaime'): u('Jaime Fernandez del Rio'),
     u('janani'): u('Janani Padmanabhan'),
     u('Janani'): u('Janani Padmanabhan'),
+    u('jnothman'): u('Joel Nothman'),
+    u('jona-sassenhagen'): u('Jona Sassenhagen'),
+    u('johschmitz'): u('Johannes Schmitz'),
     u('jor'): u('Joscha Reimer'),
     u('jesseengel'): u('Jesse Engel'),
     u('josef'): u('Josef Perktold'),
@@ -96,15 +108,21 @@ NAME_MAP = {
     u('lmwang'): u('Liming Wang'),
     u('loluengo'): u('Lorenzo Luengo'),
     u('lzkelley'): u('Luke Zoltan Kelley'),
+    u('Mad Physicist'): u('Joseph Fox-Rabinovitz'),
     u('mamrehn'): u('@mamrehn'),
     u('manns'): u('Martin Manns'),
     u('Mark'): u('Mark Wiebe'),
     u('mdroe'): u('Michael Droettboom'),
     u('maniteja123'): u('Maniteja Nandana'),
+    u('mseri'): u('Marcello Seri'),
     u('Matteo Visconti dOC'): u('Matteo Visconti'),
+    u('michael'): u('Michael Hirsch'),     # @scienceopen @ GH
+    u('Neil'): u('Neil Girdhar'),
     u('newman101'): u('Juha Remes'),
     u('nicola montecchio'): u('Nicola Montecchio'),
+    u('Nikolai'): u('Nikolai Nowaczyk'),
     u('nmoya'): u('Nikolas Moya'),
+    u('OctoMiao'): u('Lei Ma'),
     u('patricksnape'): u('Patrick Snape'),
     u('pbrod'): u('Per Brodtkorb'),
     u('Pete'): u('Pete Bunch'),
@@ -126,6 +144,7 @@ NAME_MAP = {
     u('tiagopereira'): u('Tiago M.D. Pereira'),
     u('tonysyu'): u('Tony S. Yu'),
     u('Travis E. Oliphant'): u('Travis Oliphant'),
+    u('Todd'): u('Todd Goodall'),   # @beyondmetis @ GH
     u('vanpact'): u('Yves-Rémi Van Eycke'),
     u('ubuntu'): u('Aldrian Obaja'),
     u('wa03'): u('Josh Lawrence'),
